"""Tests for MCPCapabilities mixin.

Covers OAuth token management, header parsing, auth chunks,
server configuration, and pending auth server lifecycle.
"""

import json
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from appkit_assistant.backend.processors.mcp_mixin import MCPCapabilities
from appkit_assistant.backend.schemas import ChunkType, MCPAuthType

# ============================================================================
# Helpers
# ============================================================================


def _make_server(
    server_id: int = 1,
    server_name: str = "TestServer",
    headers: str | None = None,
    auth_type: str | None = None,
    url: str = "https://mcp.test",
    prompt: str | None = None,
) -> MagicMock:
    server = MagicMock()
    server.id = server_id
    server.name = server_name
    server.headers = headers
    server.auth_type = auth_type
    server.url = url
    server.prompt = prompt
    return server


# ============================================================================
# Initialization
# ============================================================================


class TestInit:
    def test_defaults(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://app.test/callback",
        ):
            mcp = MCPCapabilities()
        assert mcp.mcp_processor_name == "unknown"
        assert mcp.current_user_id is None
        assert mcp.pending_auth_servers == []

    def test_custom_params(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://custom.test/cb",
            processor_name="openai",
        )
        assert mcp.mcp_processor_name == "openai"

    def test_user_id_setter(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://app.test/callback",
        ):
            mcp = MCPCapabilities()
        mcp.current_user_id = 42
        assert mcp.current_user_id == 42
        mcp.current_user_id = None
        assert mcp.current_user_id is None


# ============================================================================
# Pending auth servers
# ============================================================================


class TestPendingAuthServers:
    def test_add_server(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        server = _make_server()
        mcp.add_pending_auth_server(server)
        assert len(mcp.pending_auth_servers) == 1

    def test_add_duplicate_server(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        server = _make_server()
        mcp.add_pending_auth_server(server)
        mcp.add_pending_auth_server(server)
        assert len(mcp.pending_auth_servers) == 1

    def test_clear_servers(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        mcp.add_pending_auth_server(_make_server(1))
        mcp.add_pending_auth_server(_make_server(2))
        mcp.clear_pending_auth_servers()
        assert mcp.pending_auth_servers == []


# ============================================================================
# parse_mcp_headers
# ============================================================================


class TestParseMcpHeaders:
    def test_no_headers(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        server = _make_server(headers=None)
        assert mcp.parse_mcp_headers(server) == {}

    def test_empty_json_headers(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        server = _make_server(headers="{}")
        assert mcp.parse_mcp_headers(server) == {}

    def test_valid_headers(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        headers = json.dumps({"X-API-Key": "test123", "Accept": "application/json"})
        server = _make_server(headers=headers)
        result = mcp.parse_mcp_headers(server)
        assert result["X-API-Key"] == "test123"
        assert result["Accept"] == "application/json"

    def test_invalid_json(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        server = _make_server(headers="not-json{")
        result = mcp.parse_mcp_headers(server)
        assert result == {}


# ============================================================================
# parse_mcp_headers_with_auth
# ============================================================================


class TestParseMcpHeadersWithAuth:
    def test_no_auth_header(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        headers = json.dumps({"X-API-Key": "test"})
        server = _make_server(headers=headers)
        h, token = mcp.parse_mcp_headers_with_auth(server)
        assert h == {"X-API-Key": "test"}
        assert token is None

    def test_bearer_token(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        headers = json.dumps({"Authorization": "Bearer my-token"})
        server = _make_server(headers=headers)
        h, token = mcp.parse_mcp_headers_with_auth(server)
        assert "Authorization" not in h
        assert token == "my-token"  # noqa: S105

    def test_non_bearer_auth(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        headers = json.dumps({"Authorization": "Basic abc123"})
        server = _make_server(headers=headers)
        _h, token = mcp.parse_mcp_headers_with_auth(server)
        assert token == "Basic abc123"  # noqa: S105


# ============================================================================
# get_valid_token
# ============================================================================


class TestGetValidToken:
    @pytest.mark.asyncio
    async def test_delegates_to_token_service(self) -> None:
        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.mcp_oauth_redirect_uri",
            return_value="https://test/cb",
        ):
            mcp = MCPCapabilities()
        mock_token = MagicMock()
        mcp._mcp_token_service.get_valid_token = AsyncMock(  # noqa: SLF001
            return_value=mock_token
        )
        server = _make_server()
        result = await mcp.get_valid_token(server, user_id=1)
        assert result is mock_token


# ============================================================================
# create_auth_required_chunk
# ============================================================================


class TestCreateAuthRequiredChunk:
    @pytest.mark.asyncio
    async def test_creates_chunk(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="openai",
        )
        server = _make_server(server_id=5, server_name="GitHub")
        mcp.current_user_id = 1

        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.get_session_manager"
        ) as mock_sm:
            mock_session = MagicMock()
            mock_sm.return_value.session.return_value.__enter__ = MagicMock(
                return_value=mock_session
            )
            mock_sm.return_value.session.return_value.__exit__ = MagicMock(
                return_value=False
            )
            mcp._mcp_auth_service.build_authorization_url_with_registration = (  # noqa: SLF001
                AsyncMock(return_value=("https://auth.example.com/login", "state123"))
            )

            chunk = await mcp.create_auth_required_chunk(server)

        assert chunk.type == ChunkType.AUTH_REQUIRED
        assert chunk.chunk_metadata["server_id"] == "5"
        assert chunk.chunk_metadata["server_name"] == "GitHub"
        assert chunk.chunk_metadata["auth_url"] == "https://auth.example.com/login"
        assert chunk.chunk_metadata["state"] == "state123"

    @pytest.mark.asyncio
    async def test_handles_error(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="openai",
        )
        server = _make_server()

        with patch(
            "appkit_assistant.backend.processors.mcp_mixin.get_session_manager",
            side_effect=RuntimeError("db error"),
        ):
            chunk = await mcp.create_auth_required_chunk(server)

        assert chunk.type == ChunkType.AUTH_REQUIRED
        assert chunk.chunk_metadata["auth_url"] == ""


# ============================================================================
# yield_pending_auth_chunks
# ============================================================================


class TestYieldPendingAuthChunks:
    @pytest.mark.asyncio
    async def test_yields_for_each_server(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="openai",
        )
        mcp.add_pending_auth_server(_make_server(1, "Server1"))
        mcp.add_pending_auth_server(_make_server(2, "Server2"))

        with patch.object(
            mcp,
            "create_auth_required_chunk",
            new_callable=AsyncMock,
            side_effect=[
                MagicMock(type=ChunkType.AUTH_REQUIRED),
                MagicMock(type=ChunkType.AUTH_REQUIRED),
            ],
        ):
            chunks = [c async for c in mcp.yield_pending_auth_chunks()]

        assert len(chunks) == 2

    @pytest.mark.asyncio
    async def test_empty_pending(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="openai",
        )
        chunks = [c async for c in mcp.yield_pending_auth_chunks()]
        assert chunks == []


# ============================================================================
# configure_mcp_servers_with_tokens
# ============================================================================


class TestConfigureMcpServersWithTokens:
    @pytest.mark.asyncio
    async def test_empty_servers(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="test",
        )
        configs, prompt = await mcp.configure_mcp_servers_with_tokens(None, None)
        assert configs == []
        assert prompt == ""

    @pytest.mark.asyncio
    async def test_server_without_oauth(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="test",
        )
        headers = json.dumps({"X-API-Key": "key"})
        server = _make_server(headers=headers, auth_type=None, prompt="Use search")
        configs, prompt = await mcp.configure_mcp_servers_with_tokens([server], 1)
        assert len(configs) == 1
        assert configs[0]["name"] == "TestServer"
        assert configs[0]["url"] == "https://mcp.test"
        assert configs[0]["headers"]["X-API-Key"] == "key"
        assert "Use search" in prompt

    @pytest.mark.asyncio
    async def test_oauth_with_valid_token(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="test",
        )
        server = _make_server(
            auth_type=MCPAuthType.OAUTH_DISCOVERY,
            headers="{}",
        )
        mock_token = MagicMock()
        mock_token.access_token = "oauth-token"  # noqa: S105
        mcp._mcp_token_service.get_valid_token = AsyncMock(  # noqa: SLF001
            return_value=mock_token
        )

        configs, _ = await mcp.configure_mcp_servers_with_tokens([server], 1)
        assert configs[0]["headers"]["Authorization"] == "Bearer oauth-token"
        assert len(mcp.pending_auth_servers) == 0

    @pytest.mark.asyncio
    async def test_oauth_no_token_adds_pending(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="test",
        )
        server = _make_server(
            auth_type=MCPAuthType.OAUTH_DISCOVERY,
            headers="{}",
        )
        mcp._mcp_token_service.get_valid_token = AsyncMock(  # noqa: SLF001
            return_value=None
        )

        configs, _ = await mcp.configure_mcp_servers_with_tokens([server], 1)
        assert len(mcp.pending_auth_servers) == 1
        assert "Authorization" not in configs[0]["headers"]

    @pytest.mark.asyncio
    async def test_multiple_prompts_joined(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="test",
        )
        s1 = _make_server(1, "S1", headers="{}", prompt="prompt1")
        s2 = _make_server(2, "S2", headers="{}", prompt="prompt2")
        _, prompt = await mcp.configure_mcp_servers_with_tokens([s1, s2], None)
        assert "- prompt1" in prompt
        assert "- prompt2" in prompt

    @pytest.mark.asyncio
    async def test_no_prompts(self) -> None:
        mcp = MCPCapabilities(
            oauth_redirect_uri="https://app.test/cb",
            processor_name="test",
        )
        server = _make_server(headers="{}", prompt=None)
        _, prompt = await mcp.configure_mcp_servers_with_tokens([server], None)
        assert prompt == ""



import uuid
from enum import StrEnum
from typing import Any

from pydantic import BaseModel
from sqlmodel import Field


class ChunkType(StrEnum):
    """Enum for chunk types."""

    TEXT = "text"  # default
    ANNOTATION = "annotation"  # for text annotations
    IMAGE = "image"
    IMAGE_PARTIAL = "image_partial"  # for streaming image generation
    THINKING = "thinking"  # when the model is "thinking" / reasoning
    THINKING_RESULT = "thinking_result"  # when the "thinking" is done
    ACTION = "action"  # when the user needs to take action
    TOOL_RESULT = "tool_result"  # result from a tool
    TOOL_CALL = "tool_call"  # calling a tool
    PROCESSING = "processing"  # file processing status
    COMPLETION = "completion"  # when response generation is complete
    AUTH_REQUIRED = "auth_required"  # user needs to authenticate (MCP)
    ERROR = "error"  # when an error occurs
    LIFECYCLE = "lifecycle"
    MCP_APP_VIEW = "mcp_app_view"  # MCP App interactive view


class ProcessingStatistics(BaseModel):
    """Statistics about the processing request."""

    input_tokens: int | None = None
    output_tokens: int | None = None
    tool_uses: dict[str, int] = {}
    model: str | None = None
    processor: str | None = None


class Chunk(BaseModel):
    """Model for text chunks."""

    type: ChunkType
    text: str
    chunk_metadata: dict[str, str | None] = {}
    statistics: ProcessingStatistics | None = None


class ThreadStatus(StrEnum):
    """Enum for thread status."""

    NEW = "new"
    ACTIVE = "active"
    IDLE = "idle"
    WAITING = "waiting"
    ERROR = "error"
    DELETED = "deleted"
    ARCHIVED = "archived"


class MessageType(StrEnum):
    """Enum for message types."""

    HUMAN = "human"
    SYSTEM = "system"
    ASSISTANT = "assistant"
    TOOL_USE = "tool_use"
    ERROR = "error"
    INFO = "info"
    WARNING = "warning"
    MCP_APP = "mcp_app"  # MCP App interactive view


class Message(BaseModel):
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    text: str
    original_text: str | None = None  # To store original text if edited
    editable: bool = False
    type: MessageType
    done: bool = False
    attachments: list[str] = []  # List of filenames for display
    annotations: list[str] = []  # List of file citations/annotations
    mcp_app_views: list["McpAppViewData"] = []  # MCP App views for this message


class ThinkingType(StrEnum):
    REASONING = "reasoning"
    TOOL_CALL = "tool_call"
    PROCESSING = "processing"


class ThinkingStatus(StrEnum):
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    ERROR = "error"


class Thinking(BaseModel):
    type: ThinkingType
    id: str  # reasoning_session_id or tool_id
    text: str
    status: ThinkingStatus = ThinkingStatus.IN_PROGRESS
    tool_name: str | None = None
    parameters: str | None = None
    result: str | None = None
    error: str | None = None


class AIModel(BaseModel):
    id: str
    text: str
    icon: str = "codesandbox"
    stream: bool = False
    tenant_key: str = ""
    project_id: int = 0
    model: str = "default"
    temperature: float = 0.05
    supports_tools: bool = False
    supports_attachments: bool = False
    supports_search: bool = False
    supports_skills: bool = False
    keywords: list[str] = []
    requires_role: str | None = None
    active: bool = True


class Suggestion(BaseModel):
    prompt: str
    icon: str = ""


class UploadedFile(BaseModel):
    """Model for tracking uploaded files in the composer."""

    filename: str
    file_path: str
    size: int = 0


class ThreadModel(BaseModel):
    thread_id: str
    title: str = ""
    active: bool = False
    state: ThreadStatus = ThreadStatus.NEW
    prompt: str | None = ""
    messages: list[Message] = []
    ai_model: str = ""
    mcp_server_ids: list[int] = []
    skill_openai_ids: list[str] = []


class MCPAuthType(StrEnum):
    """Enum for MCP server authentication types."""

    NONE = "none"
    API_KEY = "api_key"
    OAUTH_DISCOVERY = "oauth_discovery"


class CommandDefinition(BaseModel):
    """Model for slash command definitions in the command palette."""

    id: str  # Unique command identifier (e.g., "teach", "evaluate")
    label: str  # Display label (e.g., "/teach")
    description: str = ""  # Optional description shown in palette
    icon: str = ""  # Optional icon name
    is_editable: bool = False  # True only for user's own prompts
    user_id: int = 0  # Owner user ID for prompts
    mcp_server_ids: list[int] = []  # MCP servers to auto-select


class McpAppToolInfo(BaseModel):
    """Metadata for an MCP tool that has a UI (App) view."""

    tool_name: str
    resource_uri: str
    visibility: list[str] = []
    server_id: int
    server_label: str
    input_schema: dict[str, Any] = {}


class McpAppResource(BaseModel):
    """Resource fetched from an MCP server for App rendering."""

    uri: str
    html_content: str
    csp: dict[str, str] | None = None
    permissions: dict[str, bool] | None = None
    prefers_border: bool | None = None


class McpAppViewData(BaseModel):
    """Data for rendering an MCP App view in the conversation."""

    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    server_id: int
    server_name: str
    resource_uri: str
    tool_name: str
    tool_input: dict[str, Any] = {}
    tool_result: dict[str, Any] | None = None
    html_content: str | None = None
    csp: dict[str, str] | None = None
    permissions: dict[str, bool] | None = None
    prefers_border: bool | None = None


# Resolve forward reference in Message
Message.model_rebuild()



"""
OpenAI processor for generating AI responses using OpenAI's API.
"""

import logging
from abc import ABC, abstractmethod
from collections.abc import AsyncGenerator
from typing import Any

from openai import AsyncOpenAI

from appkit_assistant.backend.database.models import (
    MCPServer,
)
from appkit_assistant.backend.processors.processor_base import ProcessorBase
from appkit_assistant.backend.schemas import (
    AIModel,
    Chunk,
    Message,
)

logger = logging.getLogger(__name__)


class BaseOpenAIProcessor(ProcessorBase, ABC):
    """Base class for OpenAI processors with common initialization and utilities."""

    def __init__(
        self,
        models: dict[str, AIModel],
        api_key: str | None = None,
        base_url: str | None = None,
        on_azure: bool = False,
        processor_name: str | None = None,
    ) -> None:
        """Initialize the base OpenAI processor.

        Args:
            models: Dictionary of supported AI models
            api_key: API key for OpenAI/Azure OpenAI
            base_url: Base URL for the API
            on_azure: Whether to use Azure OpenAI
            processor_name: Optional processor name
        """
        super().__init__(processor_name=processor_name)
        self.api_key = api_key
        self.base_url = base_url
        self.models = models
        self.on_azure = on_azure
        self.client = None

        if self.api_key and self.base_url and self.on_azure:
            self.client = AsyncOpenAI(
                api_key=self.api_key,
                base_url=f"{self.base_url}/openai/v1",
                default_query={"api-version": "preview"},
            )
        elif self.api_key and self.base_url:
            self.client = AsyncOpenAI(api_key=self.api_key, base_url=self.base_url)
        elif self.api_key:
            self.client = AsyncOpenAI(api_key=self.api_key)
        else:
            logger.warning("No API key found. Processor will not work.")

    @abstractmethod
    async def process(
        self,
        messages: list[Message],
        model_id: str,
        files: list[str] | None = None,
        mcp_servers: list[MCPServer] | None = None,
        payload: dict[str, Any] | None = None,
    ) -> AsyncGenerator[Chunk, None]:
        """Process messages and generate AI response chunks."""

    def get_supported_models(self) -> dict[str, AIModel]:
        """Return supported models if API key is available."""
        return self.models if self.api_key else {}



"""Message processing mixin for ThreadState.

Core message submission, streaming, batching, and persistence pipeline.
"""

import asyncio
import json
import logging
import re
import time
from collections.abc import AsyncGenerator
from typing import Any

import reflex as rx

from appkit_assistant.backend.database.models import (
    MCPServer,
    ThreadStatus,
)
from appkit_assistant.backend.database.repositories import (
    user_prompt_repo,
)
from appkit_assistant.backend.model_manager import ModelManager
from appkit_assistant.backend.schemas import (
    Chunk,
    ChunkType,
    McpAppToolInfo,
    Message,
    MessageType,
)
from appkit_assistant.backend.services import file_manager
from appkit_assistant.backend.services.mcp_apps_service import McpAppsService
from appkit_assistant.backend.services.response_accumulator import (
    ResponseAccumulator,
)
from appkit_assistant.backend.services.thread_service import ThreadService
from appkit_assistant.state.thread_list_state import ThreadListState
from appkit_commons.database.session import get_asyncdb_session
from appkit_user.authentication.states import UserSession

logger = logging.getLogger(__name__)

# Minimum interval between UI state flushes during streaming
_FLUSH_INTERVAL_S = 0.1  # 100 ms


class MessageProcessingMixin:
    """Mixin for message submission, streaming, and persistence.

    Expects state vars: ``processing``, ``cancellation_requested``,
    ``messages``, ``prompt``, ``thinking_items``, ``image_chunks``,
    ``show_thinking``, ``current_activity``, ``uploaded_files``,
    ``selected_mcp_servers``, ``selected_skills``, ``web_search_enabled``,
    ``with_thread_list``, ``_thread``, ``_skip_user_message``,
    ``_pending_file_cleanup``, ``_cancel_event``, ``_current_user_id``.
    """

    @rx.event(background=True)
    async def submit_message(self) -> AsyncGenerator[Any, Any]:
        """Submit a message and process the response."""
        yield rx.call_script("""
            const textarea = document.getElementById('composer-area');
            if (textarea) {
                textarea.value = '';
                textarea.style.height = 'auto';
                textarea.style.height = textarea.scrollHeight + 'px';
            }
        """)
        await self._process_message()

    @rx.event
    def request_cancellation(self) -> None:
        """Signal that the current processing should be cancelled."""
        self.cancellation_requested = True
        if self._cancel_event:
            self._cancel_event.set()
            logger.info("Cancellation requested by user")

    # ------------------------------------------------------------------
    # Core pipeline
    # ------------------------------------------------------------------

    async def _process_message(self) -> None:
        """Process the current message and stream the response.

        Uses time-based batching to reduce state lock contention:
        chunks are buffered and flushed to the UI periodically.
        """
        logger.debug("Processing message: %s", self.prompt)

        start = await self._begin_message_processing()
        if not start:
            return
        (
            current_prompt,
            selected_model,
            mcp_servers,
            file_paths,
            is_new_thread,
        ) = start

        processor = ModelManager().get_processor_for_model(selected_model)
        if not processor:
            await self._stop_processing_with_error(
                f"Keinen Adapter für das Modell gefunden: {selected_model}"
            )
            return

        async with self:
            user_session: UserSession = await self.get_state(UserSession)
            user_id = user_session.user.user_id if user_session.user else None

            logger.debug(
                "Pre-save check: is_new=%s, files=%d, user=%s",
                is_new_thread,
                len(file_paths) if file_paths else 0,
                user_id,
            )

            accumulator = ResponseAccumulator()
            accumulator.attach_messages_ref(self.messages)

            self.uploaded_files = []
            self._pending_file_cleanup = file_paths
            self._cancel_event = asyncio.Event()

        # Save thread to DB if new and has files
        if is_new_thread and file_paths and user_id:
            async with self:
                self._thread.state = ThreadStatus.ACTIVE
                self._thread.mcp_server_ids = [
                    s.id for s in self.selected_mcp_servers if s.id
                ]
                thread_copy = self._thread.model_copy()
            await ThreadService().save_thread(thread_copy, user_id)
            logger.debug(
                "Saved new thread %s to DB",
                thread_copy.thread_id,
            )

        # Discover UI tools for MCP Apps injection
        ui_tool_registry = await self._discover_ui_tools(mcp_servers, user_id)
        pending_tool_info: dict[str, dict] = {}

        first_response_received = False
        try:
            skill_ids = [s.openai_id for s in self.selected_skills]
            payload = {
                "thread_uuid": self._thread.thread_id,
                **({"web_search_enabled": True} if self.web_search_enabled else {}),
            }

            if skill_ids and self.selected_model_supports_skills:
                payload["skill_openai_ids"] = skill_ids

            chunk_buffer: list[Chunk] = []
            last_flush = time.monotonic()
            immediate_types = {
                ChunkType.COMPLETION,
                ChunkType.ERROR,
                ChunkType.AUTH_REQUIRED,
            }

            async for chunk in processor.process(
                self.messages,
                selected_model,
                files=file_paths or None,
                mcp_servers=mcp_servers,
                payload=payload,
                user_id=user_id,
                cancellation_token=self._cancel_event,
            ):
                chunk_buffer.append(chunk)

                now = time.monotonic()
                needs_flush = chunk.type in immediate_types or (
                    (now - last_flush) >= _FLUSH_INTERVAL_S
                )

                if needs_flush:
                    first_response_received = await self._flush_chunk_buffer(
                        chunks=chunk_buffer,
                        accumulator=accumulator,
                        current_prompt=current_prompt,
                        is_new_thread=is_new_thread,
                        first_response_received=(first_response_received),
                        ui_tool_registry=ui_tool_registry,
                        pending_tool_info=pending_tool_info,
                    )
                    chunk_buffer.clear()
                    last_flush = now

            # Flush remaining
            if chunk_buffer:
                first_response_received = await self._flush_chunk_buffer(
                    chunks=chunk_buffer,
                    accumulator=accumulator,
                    current_prompt=current_prompt,
                    is_new_thread=is_new_thread,
                    first_response_received=first_response_received,
                    ui_tool_registry=ui_tool_registry,
                    pending_tool_info=pending_tool_info,
                )

            await self._finalize_successful_response(accumulator)

        except Exception as ex:
            await self._handle_process_error(
                ex=ex,
                current_prompt=current_prompt,
                is_new_thread=is_new_thread,
                first_response_received=first_response_received,
            )

        finally:
            await self._finalize_processing()

    @staticmethod
    async def _discover_ui_tools(
        mcp_servers: list[MCPServer],
        user_id: int | str | None,
    ) -> dict[str, McpAppToolInfo]:
        """Discover UI-enabled tools from MCP servers."""
        registry: dict[str, McpAppToolInfo] = {}
        if not mcp_servers:
            return registry
        apps = McpAppsService()
        for server in mcp_servers:
            tools = await apps.discover_ui_tools(server, user_id or 0)
            for tool in tools:
                registry[tool.tool_name] = tool
        if registry:
            logger.info(
                "Discovered %d UI tools for MCP Apps injection",
                len(registry),
            )
        return registry

    # ------------------------------------------------------------------
    # Prompt parsing
    # ------------------------------------------------------------------

    def _parse_prompt_segments(self, prompt: str) -> list[dict]:
        """Parse prompt into text and command segments.

        Identifies command patterns (/command_name) and splits text into
        segments of type ``"text"`` or ``"command"``.
        """
        segments: list[dict] = []
        pattern = r"(?:^|(?<=\s))/([a-zA-Z0-9-]+)"

        last_end = 0
        for match in re.finditer(pattern, prompt):
            text_before = prompt[last_end : match.start()].strip()
            if text_before:
                segments.append({"type": "text", "content": text_before})
            segments.append({"type": "command", "handle": match.group(1)})
            last_end = match.end()

        if last_end < len(prompt):
            text_after = prompt[last_end:].strip()
            if text_after:
                segments.append({"type": "text", "content": text_after})

        return segments

    async def _load_command_prompt_text(
        self, user_id: int, command_handle: str
    ) -> str | None:
        """Load the prompt text for a command from the database."""
        try:
            handle = command_handle.lstrip("/")
            async with get_asyncdb_session() as session:
                prompt = await user_prompt_repo.find_latest_accessible_by_handle(
                    session, user_id, handle
                )
                if prompt:
                    logger.debug(
                        "Loaded command prompt for handle '%s'",
                        handle,
                    )
                    return prompt.prompt_text

                logger.debug(
                    "Command prompt not found for handle '%s'",
                    handle,
                )
        except Exception as e:
            logger.error(
                "Error loading command prompt for %s: %s",
                command_handle,
                e,
            )
        return None

    async def _resolve_command_segments(
        self, segments: list[dict], user_id: int
    ) -> None:
        """Resolve command handles to text prompts from DB."""
        for segment in segments:
            if segment["type"] == "command":
                try:
                    text = await self._load_command_prompt_text(
                        user_id, segment["handle"]
                    )
                    segment["resolved_text"] = text
                    if text:
                        logger.debug(
                            "Resolved command %s to prompt text",
                            segment["handle"],
                        )
                    else:
                        logger.warning(
                            "Command %s not resolved for user %d",
                            segment["handle"],
                            user_id,
                        )
                except Exception as e:
                    logger.error(
                        "Error resolving command %s: %s",
                        segment["handle"],
                        e,
                    )

    def _create_user_messages(
        self,
        prompt: str,
        segments: list[dict],
        attachments: list[str],
    ) -> list[Message]:
        """Create message objects from parsed segments."""
        if not segments:
            return [
                Message(
                    text=prompt,
                    type=MessageType.HUMAN,
                    attachments=attachments,
                )
            ]

        messages: list[Message] = []
        for i, segment in enumerate(segments):
            if segment["type"] == "text":
                atts = attachments if i == 0 else []
                messages.append(
                    Message(
                        text=segment["content"],
                        type=MessageType.HUMAN,
                        attachments=atts,
                    )
                )
            elif segment["type"] == "command":
                valid_text = segment.get("resolved_text")
                if valid_text:
                    messages.append(Message(text=valid_text, type=MessageType.HUMAN))

        logger.debug(
            "Created %d messages from prompt segments",
            len(messages),
        )
        return messages

    # ------------------------------------------------------------------
    # Processing phases
    # ------------------------------------------------------------------

    async def _begin_message_processing(
        self,
    ) -> tuple[str, str, list[MCPServer], list[str], bool] | None:
        """Prepare state for sending a message. Returns None if no-op.

        Phases:
        1. Read state under lock (prompt, flags, files)
        2. Resolve /commands via DB outside the lock
        3. Re-acquire lock to set processing state and build messages
        """
        # Phase 1
        async with self:
            current_prompt = self.prompt.strip()
            if self.processing or not current_prompt:
                return None

            is_new_thread = self._thread.state == ThreadStatus.NEW
            file_paths = [f.file_path for f in self.uploaded_files]
            attachment_names = [f.filename for f in self.uploaded_files]
            skip_user_message = self._skip_user_message
            user_id = int(self.current_user_id or 0)

        # Phase 2
        segments: list[dict] = []
        if not skip_user_message:
            segments = self._parse_prompt_segments(current_prompt)
            await self._resolve_command_segments(segments, user_id)

        # Phase 3
        async with self:
            if self.processing:
                return None

            self.processing = True
            self.image_chunks = []
            self.thinking_items = []
            self.prompt = ""

            logger.debug(
                "Begin processing: is_new=%s, files=%d, file_paths=%s",
                is_new_thread,
                len(self.uploaded_files),
                file_paths,
            )

            if skip_user_message:
                self._skip_user_message = False
            else:
                new_messages = self._create_user_messages(
                    current_prompt, segments, attachment_names
                )
                self.messages.extend(new_messages)

            self.messages.append(Message(text="", type=MessageType.ASSISTANT))

            selected_model = self.get_selected_model
            if not selected_model:
                self._add_error_message("Kein Chat-Modell ausgewählt")
                self.processing = False
                return None

            return (
                current_prompt,
                selected_model,
                self.selected_mcp_servers,
                file_paths,
                is_new_thread,
            )

    async def _stop_processing_with_error(self, error_msg: str) -> None:
        """Stop processing and show an error message."""
        async with self:
            self._add_error_message(error_msg)
            self.processing = False

    async def _flush_chunk_buffer(
        self,
        *,
        chunks: list[Chunk],
        accumulator: ResponseAccumulator,
        current_prompt: str,
        is_new_thread: bool,
        first_response_received: bool,
        ui_tool_registry: dict[str, McpAppToolInfo] | None = None,
        pending_tool_info: dict[str, dict] | None = None,
    ) -> bool:
        """Process buffered chunks and sync UI state.

        Returns updated ``first_response_received``.
        """
        async with self:
            for chunk in chunks:
                self._track_tool_call_info(chunk, ui_tool_registry, pending_tool_info)
                accumulator.process_chunk(chunk)
                self._inject_mcp_app_view(
                    chunk, accumulator, ui_tool_registry, pending_tool_info
                )

            self.thinking_items = list(accumulator.thinking_items)
            self.current_activity = accumulator.current_activity
            if accumulator.show_thinking:
                self.show_thinking = True
            if accumulator.image_chunks:
                self.image_chunks = list(accumulator.image_chunks)

            if accumulator.auth_required:
                self._handle_auth_required_from_accumulator(accumulator)

            if not first_response_received and is_new_thread:
                has_text = any(c.type == ChunkType.TEXT for c in chunks)
                if has_text:
                    first_response_received = True
                    self._thread.state = ThreadStatus.ACTIVE
                    if self._thread.title in {"", "Neuer Chat"}:
                        self._thread.title = current_prompt[:100]
                    if self.with_thread_list:
                        await self._notify_thread_created()

            return first_response_received

    @staticmethod
    def _track_tool_call_info(
        chunk: Chunk,
        ui_tool_registry: dict[str, McpAppToolInfo] | None,
        pending_tool_info: dict[str, dict] | None,
    ) -> None:
        """Track TOOL_CALL info (tool_name + server_label) by tool_id."""
        if ui_tool_registry is None or chunk.type != ChunkType.TOOL_CALL:
            return
        tool_id = chunk.chunk_metadata.get("tool_id", "")
        tool_name = chunk.chunk_metadata.get("tool_name", "")
        server_label = chunk.chunk_metadata.get("server_label", "")
        bare_name = tool_name.split(".", 1)[-1] if "." in tool_name else tool_name
        if (
            tool_id
            and bare_name
            and bare_name not in ("Unknown", "")
            and pending_tool_info is not None
        ):
            pending_tool_info.setdefault(
                tool_id,
                {"tool_name": bare_name, "server_label": server_label},
            )

    @staticmethod
    def _inject_mcp_app_view(
        chunk: Chunk,
        accumulator: ResponseAccumulator,
        ui_tool_registry: dict[str, McpAppToolInfo] | None,
        pending_tool_info: dict[str, dict] | None,
    ) -> None:
        """After TOOL_RESULT: inject MCP_APP_VIEW chunk if tool has UI."""
        if not ui_tool_registry or chunk.type != ChunkType.TOOL_RESULT:
            return
        err_val = chunk.chunk_metadata.get("error")
        if err_val is True or err_val == "True":
            return
        tool_id = chunk.chunk_metadata.get("tool_id", "")
        info = (
            pending_tool_info.pop(tool_id, {}) if pending_tool_info is not None else {}
        )
        tool_name = info.get("tool_name", "")
        tool_info = ui_tool_registry.get(tool_name)
        if not tool_info:
            return
        view_chunk = Chunk(
            type=ChunkType.MCP_APP_VIEW,
            text="",
            chunk_metadata={
                "server_id": str(tool_info.server_id),
                "server_name": tool_info.server_label,
                "resource_uri": tool_info.resource_uri,
                "tool_name": tool_info.tool_name,
                "tool_input": "{}",
                "tool_result": json.dumps(
                    {
                        "content": [{"type": "text", "text": chunk.text}],
                        "isError": False,
                    }
                ),
            },
        )
        accumulator.process_chunk(view_chunk)
        logger.debug(
            "Injected MCP_APP_VIEW for tool %s (server %s)",
            tool_name,
            tool_info.server_label,
        )

    async def _finalize_successful_response(
        self, accumulator: ResponseAccumulator
    ) -> None:
        """Finalize state after a successful full response.

        DB persistence happens outside the state lock.
        """
        async with self:
            self.show_thinking = False
            self.thinking_items = list(accumulator.thinking_items)
            self._thread.messages = list(self.messages)
            self._thread.ai_model = self.selected_model
            self._thread.mcp_server_ids = [
                s.id for s in self.selected_mcp_servers if s.id
            ]
            self._thread.skill_openai_ids = [s.openai_id for s in self.selected_skills]

            # Embed all MCP App views into the message for persistence
            # (only once, after response is complete)
            if accumulator.mcp_app_views and self.messages:
                for msg in reversed(self.messages):
                    if msg.type == MessageType.ASSISTANT:
                        msg.mcp_app_views = list(accumulator.mcp_app_views)
                        break

            user_session: UserSession = await self.get_state(UserSession)
            user_id = user_session.user.user_id if user_session.user else None
            thread_copy = self._thread.model_copy()

        if user_id:
            await ThreadService().save_thread(thread_copy, user_id)

    async def _handle_process_error(
        self,
        *,
        ex: Exception,
        current_prompt: str,
        is_new_thread: bool,
        first_response_received: bool,
    ) -> None:
        """Handle failures during streaming and persist error state."""
        async with self:
            self._thread.state = ThreadStatus.ERROR

            if self.messages and self.messages[-1].type == MessageType.ASSISTANT:
                self.messages.pop()
            self.messages.append(Message(text=str(ex), type=MessageType.ERROR))

            if is_new_thread and not first_response_received:
                if self._thread.title in {"", "Neuer Chat"}:
                    self._thread.title = current_prompt[:100]
                if self.with_thread_list:
                    await self._notify_thread_created()

            self._thread.messages = list(self.messages)
            self._thread.mcp_server_ids = [
                s.id for s in self.selected_mcp_servers if s.id
            ]

            user_session: UserSession = await self.get_state(UserSession)
            user_id = user_session.user.user_id if user_session.user else None
            thread_copy = self._thread.model_copy()

        if user_id:
            await ThreadService().save_thread(thread_copy, user_id)

    async def _finalize_processing(self) -> None:
        """Mark processing done and close out the last message."""
        pending_files: list[str] = []
        async with self:
            if self.messages:
                self.messages[-1].done = True
            self.processing = False
            self.cancellation_requested = False
            self.current_activity = ""
            self._cancel_event = None
            self.mcp_app_views = []  # Clear state views after embedding

            if self._pending_file_cleanup:
                pending_files = list(self._pending_file_cleanup)
                self._pending_file_cleanup = []

        if pending_files:
            file_manager.cleanup_uploaded_files(pending_files)

    def _add_error_message(self, error_msg: str) -> None:
        """Add an error message to the conversation."""
        logger.error(error_msg)
        self.messages.append(Message(text=error_msg, type=MessageType.ERROR))

    async def _notify_thread_created(self) -> None:
        """Notify ThreadListState that a new thread was created.

        Called from within an ``async with self`` block.
        """
        threadlist_state: ThreadListState = await self.get_state(ThreadListState)
        await threadlist_state.add_thread(self._thread)



import reflex as rx

import appkit_mantine as mn
from appkit_assistant.backend.schemas import (
    Message,
    MessageType,
    Thinking,
    ThinkingStatus,
    ThinkingType,
)
from appkit_assistant.components.mcp_app_view import mcp_app_view
from appkit_assistant.state.thread_state import (
    ThreadState,
)
from appkit_ui.components.collapsible import collapsible
from appkit_ui.components.dialogs import delete_dialog

# Approximate character threshold for 6 lines of text
USER_MESSAGE_COLLAPSE_THRESHOLD = 300

message_styles = {
    "spacing": "4",
    "width": "100%",
    "max_width": "880px",
    "margin_top": "24px",
    "margin_left": "auto",
    "margin_right": "auto",
}


class AuthCardComponent:
    """Component for displaying MCP OAuth authentication cards."""

    @staticmethod
    def render() -> rx.Component:
        """Render the auth required card when authentication is needed."""
        return rx.cond(
            ThreadState.show_auth_card,
            rx.hstack(
                rx.avatar(
                    fallback="🔐",
                    size="3",
                    variant="soft",
                    radius="full",
                    margin_top="16px",
                ),
                rx.card(
                    rx.vstack(
                        rx.hstack(
                            rx.text(
                                "Anmeldung erforderlich",
                                weight="bold",
                                size="3",
                            ),
                            spacing="2",
                        ),
                        rx.text(
                            rx.text.span(ThreadState.pending_auth_server_name),
                            " benötigt Ihre Anmeldung, um fortzufahren.",
                            size="2",
                            color=rx.color("gray", 11),
                        ),
                        rx.hstack(
                            rx.button(
                                "Anmelden",
                                on_click=ThreadState.start_mcp_oauth,
                                color_scheme="amber",
                                variant="solid",
                            ),
                            rx.button(
                                "Abbrechen",
                                on_click=ThreadState.dismiss_auth_card,
                                color_scheme="gray",
                                variant="solid",
                            ),
                            spacing="3",
                            margin_top="2",
                        ),
                        spacing="3",
                        padding="4",
                    ),
                    size="2",
                    variant="surface",
                    max_width="400px",
                    margin_top="16px",
                ),
                style=message_styles,
            ),
            rx.fragment(),
        )


class MessageActionsBar:
    """Component for message action buttons (copy, download, retry)."""

    @staticmethod
    def render(message: Message) -> rx.Component:
        return rx.hstack(
            rx.tooltip(
                mn.action_icon(
                    rx.icon("copy", size=13),
                    on_click=ThreadState.copy_message(message.text),
                    variant="subtle",
                    size="sm",
                    color="gray",
                ),
                content="Kopieren",
            ),
            rx.tooltip(
                mn.action_icon(
                    rx.icon("download", size=13),
                    on_click=ThreadState.download_message(message.text, message.id),
                    variant="subtle",
                    size="sm",
                    color="gray",
                ),
                content="Herunterladen",
            ),
            rx.tooltip(
                delete_dialog(
                    title="Nachricht löschen",
                    content="diese Nachricht",
                    on_click=ThreadState.delete_message(message.id),
                    icon_button=True,
                    variant="subtle",
                    size="sm",
                    color="gray",
                ),
                content="Löschen",
            ),
            rx.cond(
                (message.type == MessageType.ASSISTANT)
                | (message.type == MessageType.ERROR),
                rx.tooltip(
                    mn.action_icon(
                        rx.cond(
                            ThreadState.processing,
                            rx.spinner(size="1"),
                            rx.icon("refresh-cw", size=13),
                        ),
                        on_click=ThreadState.retry_message(message.id),
                        variant="subtle",
                        size="sm",
                        color="gray",
                        disabled=ThreadState.processing,
                    ),
                    content="Erneut generieren (folgende Nachrichten werden entfernt)",
                ),
                rx.fragment(),
            ),
            spacing="3",
            margin_top="-9px",
            margin_left="9px",
        )


def _render_annotation(ann: str) -> rx.Component:
    return rx.cond(
        ann.contains("http"),
        rx.badge(
            rx.link(
                rx.hstack(
                    rx.icon("globe", size=12, flex_shrink=0),
                    rx.text(
                        ann,
                        style={
                            "overflow": "hidden",
                            "white_space": "nowrap",
                            "text_overflow": "ellipsis",
                        },
                    ),
                    align="center",
                    spacing="1",
                    width="100%",
                ),
                href=ann,
                is_external=True,
                size="1",
                width="100%",
                text_decoration="none",
                style={"overflow": "hidden"},
            ),
            size="1",
            variant="soft",
            color_scheme="gray",
            max_width="100%",
        ),
        rx.badge(
            rx.hstack(
                rx.icon("file-text", size=12, flex_shrink=0),
                rx.text(
                    ann,
                    style={
                        "overflow": "hidden",
                        "white_space": "nowrap",
                        "text_overflow": "ellipsis",
                    },
                ),
                align="center",
                spacing="1",
                width="100%",
            ),
            size="1",
            variant="soft",
            color_scheme="gray",
            max_width="100%",
        ),
    )


class MessageComponent:
    @staticmethod
    def _file_badge(filename: str) -> rx.Component:
        """Render a single file attachment badge."""
        return rx.badge(
            rx.icon("paperclip", size=12),
            filename,
            variant="soft",
            color="gray",
            size="1",
            radius="small",
        )

    @staticmethod
    def _attachments_row(attachments: list[str]) -> rx.Component:
        """Render a row of file attachment badges."""
        return rx.cond(
            attachments.length() > 0,
            rx.hstack(
                rx.foreach(attachments, MessageComponent._file_badge),
                spacing="2",
                margin_top="8px",
                justify="end",
                width="90%",
                flex_wrap="wrap",
            ),
            rx.fragment(),
        )

    @staticmethod
    def _message_text_box(message: Message) -> rx.Component:
        """Render the message text box with collapsible support for long messages."""
        # Check if message is expanded
        is_expanded = ThreadState.expanded_message_ids.contains(message.id)
        is_long = message.text.length() > USER_MESSAGE_COLLAPSE_THRESHOLD

        # Text content
        text_content = rx.text(
            message.text,
            padding="0.5em",
            white_space="pre-line",
        )

        # Animated text container - uses max-height transition
        text_container = rx.box(
            text_content,
            max_height=rx.cond(is_expanded, "2000px", "156px"),
            overflow="hidden",
            # Only apply line clamp when collapsed
            style=rx.cond(
                is_expanded,
                {
                    "transition": "max-height 0.3s ease-in-out",
                },
                {
                    "text-overflow": "ellipsis",
                    "transition": "max-height 0.4s ease-in-out",
                },
            ),
        )

        # Expand/collapse button inside the bubble
        expand_button = rx.box(
            rx.hstack(
                rx.icon(
                    rx.cond(is_expanded, "chevron-up", "chevron-down"),
                    size=14,
                    color=rx.color("gray", 9),
                    style={
                        "transition": "transform 0.2s ease-in-out",
                    },
                ),
                rx.text(
                    rx.cond(is_expanded, "Weniger anzeigen", "Mehr anzeigen"),
                    size="1",
                    color=rx.color("gray", 9),
                ),
                spacing="1",
                align="center",
                justify="center",
                width="100%",
            ),
            on_click=ThreadState.toggle_message_expanded(message.id),
            cursor="pointer",
            padding="6px",
            border_top=f"1px solid {rx.color('accent', 5)}",
            width="100%",
            _hover={"background_color": rx.color("accent", 4)},
        )

        # Message box with optional expand button
        return rx.box(
            rx.vstack(
                # Animated text container
                rx.cond(
                    is_long,
                    text_container,
                    text_content,
                ),
                # Expand/collapse button (only for long messages)
                rx.cond(
                    is_long,
                    expand_button,
                    rx.fragment(),
                ),
                spacing="0",
                width="100%",
            ),
            padding="4px",
            padding_bottom=rx.cond(is_long, "0px", "4px"),
            max_width="800px",
            background_color=rx.color_mode_cond(
                light=rx.color("accent", 3),
                dark=rx.color("accent", 3),
            ),
            border_radius="9px",
            overflow="hidden",
        )

    @staticmethod
    def human_message(message: Message) -> rx.Component:
        return rx.cond(
            ThreadState.editing_message_id == message.id,
            # Edit Mode
            rx.vstack(
                rx.text_area(
                    value=ThreadState.edited_message_content,
                    on_change=ThreadState.set_edited_message_content,
                    height="112px",
                    width="824px",
                    auto_focus=True,
                    bg=rx.color("gray", 3),
                    variant="soft",
                ),
                rx.hstack(
                    rx.button(
                        "Abbrechen",
                        on_click=ThreadState.cancel_edit,
                        variant="soft",
                        color_scheme="gray",
                    ),
                    rx.button("Senden", on_click=ThreadState.submit_edited_message),
                    justify="end",
                    width="100%",
                    spacing="2",
                ),
                style=message_styles,
                align="end",
            ),
            rx.vstack(
                rx.hstack(
                    rx.spacer(),
                    rx.vstack(
                        MessageComponent._message_text_box(message),
                        MessageComponent._attachments_row(message.attachments),
                        align="end",
                        spacing="1",
                    ),
                ),
                rx.hstack(
                    rx.spacer(),
                    rx.tooltip(
                        mn.action_icon(
                            rx.icon("pencil", size=13),
                            on_click=ThreadState.set_editing_mode(
                                message.id, message.text
                            ),
                            variant="subtle",
                            size="sm",
                            c="gray",
                        ),
                        content="Bearbeiten",
                    ),
                    rx.tooltip(
                        delete_dialog(
                            title="Nachricht löschen",
                            content="diese Nachricht",
                            on_click=ThreadState.delete_message(message.id),
                            icon_button=True,
                            variant="subtle",
                            size="sm",
                            c="gray",
                        ),
                        content="Löschen",
                    ),
                    rx.tooltip(
                        mn.action_icon(
                            rx.icon("copy", size=13),
                            on_click=ThreadState.copy_message(message.text),
                            variant="subtle",
                            size="sm",
                            color="gray",
                        ),
                        content="Kopieren",
                    ),
                    spacing="3",
                    justify="end",
                    margin_right="9px",
                ),
                align="end",
                style=message_styles,
            ),
        )

    @staticmethod
    def assistant_message(message: Message) -> rx.Component:
        """Display an assistant message with thinking content when items exist."""

        # Show thinking content only for the last assistant message
        should_show_thinking = (
            message.text == ThreadState.get_last_assistant_message_text
        ) & ThreadState.has_thinking_content

        # Main content area with all components
        content_area = rx.vstack(
            # Always rendered with conditional styling for smooth animations
            collapsible(
                rx.scroll_area(
                    rx.foreach(
                        ThreadState.thinking_items,
                        lambda item: ToolCallComponent.render(item),  # noqa: PLW0108
                    ),
                    spacing="3",
                    max_height="180px",
                    padding="9px 12px",
                    width="100%",
                    scrollbars="vertical",
                ),
                title="Denkprozess & Werkzeuge",
                info_text=(
                    f"{ThreadState.get_unique_reasoning_sessions.length()} "
                    f"Nachdenken, "
                    f"{ThreadState.get_unique_tool_calls.length()} Werkzeuge"
                ),
                show_condition=should_show_thinking,
                expanded=ThreadState.thinking_expanded,
                on_toggle=ThreadState.toggle_thinking_expanded,
            ),
            # Main message content
            rx.cond(
                message.text == "",
                rx.hstack(
                    rx.text(
                        rx.cond(
                            ThreadState.current_activity != "",
                            ThreadState.current_activity,
                            "Denke nach",
                        ),
                        class_name="shimmer-text",
                        margin_right="9px",
                    ),
                    class_name="loading",
                    height="40px",
                    color=rx.color("gray", 8),
                    background_color=rx.color("gray", 2),
                    padding="0.5em",
                    border_radius="9px",
                    margin_top="16px",
                    padding_right="18px",
                ),
                # Actual message content
                rx.box(
                    mn.markdown_preview(
                        source=message.text,
                        enable_mermaid=message.done,
                        enable_katex=message.done,
                        security_level="standard",
                        class_name="markdown",
                    ),
                    padding="0.5em",
                    margin_top="18px",
                    max_width="90%",
                ),
            ),
            # Annotations/Sources
            rx.cond(
                message.annotations.length() > 0,
                rx.vstack(
                    rx.hstack(
                        rx.icon("file-text", size=13, color=rx.color("gray", 9)),
                        rx.text(
                            "Quellen:",
                            size="1",
                            color=rx.color("gray", 9),
                        ),
                        spacing="1",
                    ),
                    rx.hstack(
                        rx.foreach(
                            message.annotations,
                            _render_annotation,
                        ),
                        spacing="2",
                        align="start",
                        flex_wrap="wrap",
                        max_width="95%",
                    ),
                    padding="6px",
                    margin_top="6px",
                    margin_bottom="6px",
                    width="90%",
                    align="start",
                ),
                rx.fragment(),
            ),
            # Actions bar
            rx.cond(
                message.done,
                MessageActionsBar.render(message),
                rx.fragment(),
            ),
            # MCP App views (per-message, persisted in chat history)
            rx.cond(
                message.mcp_app_views.length() > 0,
                rx.vstack(
                    rx.foreach(
                        message.mcp_app_views,
                        mcp_app_view,
                    ),
                    width="100%",
                    spacing="2",
                    margin_top="8px",
                ),
                rx.fragment(),
            ),
            spacing="3",
            width="100%",
        )

        return rx.hstack(
            rx.avatar(
                fallback="AI",
                size="3",
                variant="soft",
                radius="full",
                margin_top="16px",
            ),
            content_area,
            style=message_styles,
        )

    @staticmethod
    def info_message(message: str) -> rx.Component:
        return rx.hstack(
            rx.avatar(
                fallback="AI",
                size="3",
                variant="soft",
                radius="full",
                margin_top="16px",
            ),
            rx.callout(
                message,
                icon="info",
                max_width="90%",
                size="1",
                padding="0.5em",
                border_radius="9px",
                margin_top="18px",
            ),
            style=message_styles,
        )

    @staticmethod
    def error_message(message: Message) -> rx.Component:
        return rx.hstack(
            rx.avatar(
                fallback="!",
                size="3",
                variant="soft",
                radius="full",
                margin_top="16px",
                color_scheme="red",
            ),
            rx.vstack(
                rx.callout(
                    message.text,
                    icon="triangle-alert",
                    color_scheme="red",
                    max_width="100%",
                    size="1",
                    padding="0.5em",
                    border_radius="9px",
                    margin_top="18px",
                ),
                MessageActionsBar.render(message),
                width="90%",
                spacing="2",
            ),
            style=message_styles,
        )

    @staticmethod
    def system_message(message: str) -> rx.Component:
        return rx.hstack(
            rx.avatar(
                fallback="⚙",
                size="3",
                variant="soft",
                radius="full",
                margin_top="16px",
                color_scheme="gray",
            ),
            rx.callout(
                message,
                icon="info",
                color_scheme="gray",
                max_width="90%",
                size="1",
                padding="0.5em",
                border_radius="9px",
                margin_top="18px",
                # class_name="system-message-shimmer",
            ),
            style=message_styles,
        )

    @staticmethod
    def render_message(
        message: Message,
    ) -> rx.Component:
        """Render message with optional enhanced chunk-based components."""
        return rx.fragment(
            rx.match(
                message.type,
                (
                    MessageType.HUMAN,
                    MessageComponent.human_message(message),
                ),
                (
                    MessageType.ASSISTANT,
                    MessageComponent.assistant_message(message),
                ),
                (
                    MessageType.ERROR,
                    MessageComponent.error_message(message),
                ),
                (
                    MessageType.SYSTEM,
                    MessageComponent.system_message(message.text),
                ),
                MessageComponent.info_message(message.text),
            )
        )


class ToolCallComponent:
    """Component for displaying individual tool calls with green styling."""

    @staticmethod
    def render(tool_item: Thinking) -> rx.Component:
        return rx.cond(
            tool_item.type == ThinkingType.REASONING,
            ToolCallComponent._render_reasoning(tool_item),
            rx.cond(
                tool_item.type == ThinkingType.PROCESSING,
                ToolCallComponent._render_processing(tool_item),
                ToolCallComponent._render_tool_call(tool_item),
            ),
        )

    @staticmethod
    def _render_reasoning(item: Thinking) -> rx.Component:
        return rx.vstack(
            rx.text(item.text, size="1"),
            border_left=f"3px solid {rx.color('gray', 4)}",
            padding="3px 6px",
            margin_bottom="9px",
        )

    @staticmethod
    def _render_processing(item: Thinking) -> rx.Component:
        """Render file processing progress."""
        return rx.hstack(
            rx.cond(
                item.status == ThinkingStatus.COMPLETED,
                rx.icon("circle-check", size=14, color=rx.color("green", 9)),
                rx.cond(
                    item.status == ThinkingStatus.ERROR,
                    rx.icon("circle-x", size=14, color=rx.color("red", 9)),
                    rx.icon("file-up", size=14, color=rx.color("blue", 9)),
                ),
            ),
            rx.text(
                item.text,
                size="1",
                color=rx.cond(
                    item.status == ThinkingStatus.ERROR,
                    rx.color("red", 9),
                    rx.color("gray", 10),
                ),
            ),
            spacing="2",
            padding="3px 6px",
            margin_bottom="3px",
            width="100%",
        )

    @staticmethod
    def _render_tool_call(item: Thinking) -> rx.Component:
        return rx.vstack(
            rx.hstack(
                rx.icon("wrench", size=14, color=rx.color("green", 8)),
                rx.text(
                    f"Werkzeug: {item.tool_name}",
                    size="1",
                    font_weight="bold",
                    color=rx.color("blue", 9),
                ),
                rx.spacer(),
                rx.text(
                    item.id,
                    size="1",
                    color=rx.color("gray", 6),
                ),
                spacing="1",
                margin_bottom="3px",
                width="100%",
            ),
            rx.cond(
                item.text,
                rx.vstack(
                    rx.text(
                        item.text,
                        size="1",
                        color=rx.color("gray", 10),
                    ),
                    align="start",
                    width="100%",
                ),
            ),
            rx.cond(
                item.parameters,
                rx.vstack(
                    rx.text(
                        item.parameters,
                        size="1",
                        color=rx.color("blue", 9),
                        white_space="pre-wrap",
                    ),
                    align="start",
                    width="100%",
                    spacing="1",
                ),
            ),
            rx.cond(
                item.status == ThinkingStatus.COMPLETED,
                rx.scroll_area(
                    rx.text(
                        item.result,
                        size="1",
                        color=rx.color("gray", 8),
                    ),
                    max_height="60px",
                    width="95%",
                    scrollbars="vertical",
                ),
            ),
            rx.cond(
                item.status == ThinkingStatus.ERROR,
                rx.vstack(
                    rx.hstack(
                        rx.icon("shield-alert", size=14, color=rx.color("red", 10)),
                        rx.text(
                            "Fehler",
                            size="1",
                            font_weight="bold",
                            color=rx.color("red", 10),
                        ),
                        spacing="1",
                    ),
                    rx.text(
                        item.error,
                        size="1",
                        color=rx.color("red", 9),
                    ),
                    align="start",
                    width="100%",
                    spacing="1",
                ),
            ),
            padding="3px 6px",
            border_left=f"3px solid {rx.color('gray', 5)}",
            margin_bottom="9px",
            width="100%",
            spacing="2",
        )



# ruff: noqa: ARG002, SLF001, S105, S106
"""Tests for SkillState.

Covers load_user_skills, toggle_skill_enabled,
and computed vars (has_skills, enabled_skill_ids).
"""

from __future__ import annotations

from typing import Any
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from appkit_assistant.state.skill_state import SkillState

_PATCH = "appkit_assistant.state.skill_state"

_CV = SkillState.__dict__


def _unwrap(name: str):
    entry = SkillState.__dict__[name]
    return entry.fn if hasattr(entry, "fn") else entry


def _db_context(session: AsyncMock | None = None):
    s = session or AsyncMock()
    cm = AsyncMock()
    cm.__aenter__ = AsyncMock(return_value=s)
    cm.__aexit__ = AsyncMock(return_value=False)
    return cm


class _StubSkillState:
    def __init__(self) -> None:
        self.available_skills: list[dict[str, Any]] = []
        self.loading: bool = False
        self._mock_user_session: MagicMock | None = None

    async def get_state(self, cls: type) -> Any:
        return self._mock_user_session

    load_user_skills = _unwrap("load_user_skills")
    toggle_skill_enabled = _unwrap("toggle_skill_enabled")


def _make_state(
    user_id: int | None = 42,
) -> _StubSkillState:
    s = _StubSkillState()
    user_session = MagicMock()
    user_session.user_id = user_id
    s._mock_user_session = user_session
    return s


def _mock_skill(
    openai_id: str = "sk-1",
    name: str = "Skill1",
    description: str = "Desc",
    active: bool = True,
):
    s = MagicMock()
    s.openai_id = openai_id
    s.name = name
    s.description = description
    s.active = active
    return s


def _mock_selection(openai_id: str = "sk-1", enabled: bool = True):
    sel = MagicMock()
    sel.skill_openai_id = openai_id
    sel.enabled = enabled
    return sel


# ============================================================================
# Computed vars
# ============================================================================


class TestComputedVars:
    def test_has_skills_true(self) -> None:
        state = _make_state()
        state.available_skills = [{"openai_id": "sk-1"}]
        assert _CV["has_skills"].fget(state) is True

    def test_has_skills_false(self) -> None:
        state = _make_state()
        assert _CV["has_skills"].fget(state) is False

    def test_enabled_skill_ids_empty(self) -> None:
        state = _make_state()
        assert _CV["enabled_skill_ids"].fget(state) == []

    def test_enabled_skill_ids_mixed(self) -> None:
        state = _make_state()
        state.available_skills = [
            {"openai_id": "sk-1", "enabled": True},
            {"openai_id": "sk-2", "enabled": False},
            {"openai_id": "sk-3", "enabled": True},
        ]
        result = _CV["enabled_skill_ids"].fget(state)
        assert result == ["sk-1", "sk-3"]


# ============================================================================
# load_user_skills
# ============================================================================


class TestLoadUserSkills:
    @pytest.mark.asyncio
    async def test_no_user(self) -> None:
        state = _make_state(user_id=None)
        _ = [c async for c in state.load_user_skills()]
        assert state.available_skills == []
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_zero_user_id(self) -> None:
        state = _make_state(user_id=0)
        _ = [c async for c in state.load_user_skills()]
        assert state.available_skills == []

    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _make_state()
        skill = _mock_skill("sk-1", "SkillA", "A desc")
        sel = _mock_selection("sk-1", enabled=True)

        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.skill_repo") as sr,
            patch(f"{_PATCH}.user_skill_repo") as usr,
        ):
            sr.find_all_active_ordered_by_name = AsyncMock(return_value=[skill])
            usr.find_by_user_id = AsyncMock(return_value=[sel])
            _ = [c async for c in state.load_user_skills()]

        assert len(state.available_skills) == 1
        assert state.available_skills[0]["name"] == "SkillA"
        assert state.available_skills[0]["enabled"] is True
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_no_selection_defaults_false(self) -> None:
        state = _make_state()
        skill = _mock_skill("sk-1")

        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.skill_repo") as sr,
            patch(f"{_PATCH}.user_skill_repo") as usr,
        ):
            sr.find_all_active_ordered_by_name = AsyncMock(return_value=[skill])
            usr.find_by_user_id = AsyncMock(return_value=[])
            _ = [c async for c in state.load_user_skills()]

        assert state.available_skills[0]["enabled"] is False

    @pytest.mark.asyncio
    async def test_error(self) -> None:
        state = _make_state()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.skill_repo") as sr,
        ):
            sr.find_all_active_ordered_by_name = AsyncMock(
                side_effect=RuntimeError("db")
            )
            _ = [c async for c in state.load_user_skills()]
        assert state.loading is False


# ============================================================================
# toggle_skill_enabled
# ============================================================================


class TestToggleSkillEnabled:
    @pytest.mark.asyncio
    async def test_optimistic_update(self) -> None:
        state = _make_state()
        state.available_skills = [
            {
                "openai_id": "sk-1",
                "name": "S",
                "enabled": False,
            },
        ]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.user_skill_repo") as usr,
        ):
            usr.upsert = AsyncMock()
            _ = [c async for c in state.toggle_skill_enabled("sk-1", True)]
        assert state.available_skills[0]["enabled"] is True

    @pytest.mark.asyncio
    async def test_error_reverts(self) -> None:
        state = _make_state()
        state.available_skills = [
            {
                "openai_id": "sk-1",
                "name": "S",
                "enabled": False,
            },
        ]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.user_skill_repo") as usr,
        ):
            usr.upsert = AsyncMock(side_effect=RuntimeError("db"))
            _ = [c async for c in state.toggle_skill_enabled("sk-1", True)]
        # Should be reverted back to False
        assert state.available_skills[0]["enabled"] is False

    @pytest.mark.asyncio
    async def test_no_user(self) -> None:
        state = _make_state(user_id=None)
        state.available_skills = [
            {
                "openai_id": "sk-1",
                "name": "S",
                "enabled": False,
            },
        ]
        _ = [c async for c in state.toggle_skill_enabled("sk-1", True)]
        # Optimistic update happens, but no DB call
        assert state.available_skills[0]["enabled"] is True

    @pytest.mark.asyncio
    async def test_disable(self) -> None:
        state = _make_state()
        state.available_skills = [
            {
                "openai_id": "sk-1",
                "name": "S",
                "enabled": True,
            },
        ]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.user_skill_repo") as usr,
        ):
            usr.upsert = AsyncMock()
            _ = [c async for c in state.toggle_skill_enabled("sk-1", False)]
        assert state.available_skills[0]["enabled"] is False



# ruff: noqa: ARG002, SLF001, S105, S106
"""Tests for ThreadState.

Covers computed vars, initialization, thread management,
setters, toggles, clear, persistence, and load_thread.
"""

from __future__ import annotations

import uuid
from types import SimpleNamespace
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from appkit_assistant.backend.database.models import ThreadStatus
from appkit_assistant.backend.model_manager import ModelManager
from appkit_assistant.backend.schemas import (
    AIModel,
    Message,
    MessageType,
    Suggestion,
    Thinking,
    ThinkingType,
    ThreadModel,
)
from appkit_assistant.state.thread_state import ThreadState


class _Awaitable:
    """Thin awaitable wrapper — can be awaited multiple times without warnings."""

    __slots__ = ("_value",)

    def __init__(self, value: object) -> None:
        self._value = value

    def __await__(self):
        yield
        return self._value


_PATCH = "appkit_assistant.state.thread_state"

# Access computed-var descriptors via __dict__.
_CV = ThreadState.__dict__


def _unwrap(name: str):
    """Get the raw function from an EventHandler in __dict__."""
    entry = ThreadState.__dict__[name]
    return entry.fn if hasattr(entry, "fn") else entry


def _thread_model(
    thread_id: str = "t1",
    state: ThreadStatus = ThreadStatus.NEW,
    ai_model: str = "gpt-4o",
    prompt: str = "",
    messages: list[Message] | None = None,
    mcp_server_ids: list[int] | None = None,
    skill_openai_ids: list[str] | None = None,
) -> ThreadModel:
    return ThreadModel(
        thread_id=thread_id,
        title="Chat",
        state=state,
        ai_model=ai_model,
        prompt=prompt,
        messages=messages or [],
        mcp_server_ids=mcp_server_ids or [],
        skill_openai_ids=skill_openai_ids or [],
    )


def _message(
    msg_type: MessageType = MessageType.HUMAN,
    text: str = "Hello",
    msg_id: str | None = None,
) -> Message:
    return Message(
        id=msg_id or str(uuid.uuid4()),
        type=msg_type,
        text=text,
        done=True,
    )


def _ai_model(
    model_id: str = "gpt-4o",
    text: str = "GPT-4o",
    supports_tools: bool = True,
    supports_attachments: bool = True,
    supports_search: bool = False,
    supports_skills: bool = True,
) -> AIModel:
    return AIModel(
        id=model_id,
        text=text,
        supports_tools=supports_tools,
        supports_attachments=supports_attachments,
        supports_search=supports_search,
        supports_skills=supports_skills,
    )


def _thinking(
    thinking_type: ThinkingType = ThinkingType.REASONING,
    thinking_id: str = "r1",
) -> Thinking:
    return Thinking(id=thinking_id, type=thinking_type, text="thinking...")


class _StubThreadState:
    """Plain stub providing ThreadState vars for testing.

    Does NOT inherit from rx.State to avoid __setattr__ issues.
    """

    def __init__(
        self,
        *,
        authenticated: bool = True,
        user_id: str = "42",
    ) -> None:
        self._thread = _thread_model()
        self.ai_models: list[AIModel] = []
        self.selected_model: str = ""
        self.processing: bool = False
        self.cancellation_requested: bool = False
        self.messages: list[Message] = []
        self.prompt: str = ""
        self.suggestions: list[Suggestion] = []

        self.thinking_items: list[Thinking] = []
        self.image_chunks: list = []
        self.show_thinking: bool = False
        self.thinking_expanded: bool = False
        self.current_activity: str = ""

        self.uploaded_files: list = []
        self.max_file_size_mb: int = 50
        self.max_files_per_thread: int = 10

        self.editing_message_id: str | None = None
        self.edited_message_content: str = ""
        self.expanded_message_ids: list[str] = []

        self.selected_mcp_servers: list = []
        self.show_tools_modal: bool = False
        self.available_mcp_servers: list = []
        self.temp_selected_mcp_servers: list[int] = []
        self.server_selection_state: dict[int, bool] = {}

        self.selected_skills: list = []
        self.available_skills_for_selection: list = []
        self.temp_selected_skill_ids: list[str] = []
        self.skill_selection_state: dict[str, bool] = {}
        self.modal_active_tab: str = "tools"

        self.web_search_enabled: bool = False

        self.show_command_palette: bool = False
        self.filtered_commands: list = []
        self.selected_command_index: int = 0
        self.command_search_prefix: str = ""
        self.command_trigger_position: int = 0
        self.available_commands: list = []

        self.with_thread_list: bool = False

        self._initialized: bool = False
        self._current_user_id: str = ""
        self._skip_user_message: bool = False
        self._pending_file_cleanup: list[str] = []
        self._cancel_event = None

        # Test config
        self._authenticated = authenticated
        self._user_id = user_id

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        return False

    async def get_state(self, cls: type) -> MagicMock:
        name = cls.__name__
        if name == "UserSession":
            user = (
                SimpleNamespace(
                    user_id=self._user_id,
                    roles=["admin"],
                )
                if self._authenticated
                else None
            )

            mock = MagicMock()
            mock.user = user
            mock.is_authenticated = _Awaitable(self._authenticated)
            mock.authenticated_user = _Awaitable(user)
            return mock
        if name == "ThreadListState":
            mock = MagicMock()
            mock.threads = []
            mock.loading_thread_id = ""
            mock.active_thread_id = ""
            return mock
        return MagicMock()

    # Bind methods that are needed for testing
    def _setup_models(self, user) -> None:  # noqa: ARG002
        """Stub _setup_models from ModelSelectionMixin."""

    def _reset_ui_state(self) -> None:
        _unwrap("_reset_ui_state")(self)

    def _restore_mcp_selection(self, server_ids: list[int]) -> None:
        """Stub _restore_mcp_selection."""
        self.selected_mcp_servers = []
        self.temp_selected_mcp_servers = []
        self.server_selection_state = {}

    def _restore_skill_selection(self, skill_ids: list[str]) -> None:
        """Stub _restore_skill_selection."""
        self.selected_skills = []
        self.temp_selected_skill_ids = []
        self.skill_selection_state = {}

    def _clear_uploaded_files(self) -> None:
        """Stub _clear_uploaded_files."""
        self.uploaded_files = []

    def _update_command_palette(self, prompt: str) -> None:  # noqa: ARG002
        """Stub _update_command_palette."""

    async def _load_user_prompts_as_commands(self, user_id: int) -> None:  # noqa: ARG002
        """Stub _load_user_prompts_as_commands."""

    async def _stop_loading_state(self) -> None:
        """Stub _stop_loading_state."""

    async def _apply_loaded_thread(
        self, full_thread: ThreadModel, thread_id: str
    ) -> None:
        """Stub _apply_loaded_thread — mirrors real ThreadState logic."""
        self._thread = full_thread
        self.messages = full_thread.messages
        self.selected_model = full_thread.ai_model
        self.thinking_items = []
        self.image_chunks = []
        self.mcp_app_views = []
        self._ui_tool_registry = {}
        self.prompt = ""
        self.web_search_enabled = False

        model = ModelManager().get_model(full_thread.ai_model)

        if model and model.supports_tools:
            self._restore_mcp_selection(full_thread.mcp_server_ids)
        else:
            self._restore_mcp_selection([])

        if model and model.supports_skills:
            self._restore_skill_selection(full_thread.skill_openai_ids or [])
        else:
            self._restore_skill_selection([])

    def _load_config(self) -> None:
        """Stub _load_config."""

    async def initialize(self) -> None:
        """Stub initialize for patch.object compatibility."""

    @property
    def _thread_service(self) -> MagicMock:
        return self._mock_thread_service


def _make_state(*, authenticated: bool = True, user_id: str = "42") -> _StubThreadState:
    s = _StubThreadState(authenticated=authenticated, user_id=user_id)
    s._mock_thread_service = MagicMock()
    return s


# ============================================================================
# Computed vars
# ============================================================================


class TestComputedVars:
    def test_current_user_id(self) -> None:
        state = _make_state()
        state._current_user_id = "u123"
        assert _CV["current_user_id"].fget(state) == "u123"

    def test_current_user_id_empty(self) -> None:
        state = _make_state()
        assert _CV["current_user_id"].fget(state) == ""

    def test_has_suggestions_false(self) -> None:
        state = _make_state()
        assert _CV["has_suggestions"].fget(state) is False

    def test_has_suggestions_true(self) -> None:
        state = _make_state()
        state.suggestions = [Suggestion(prompt="test prompt")]
        assert _CV["has_suggestions"].fget(state) is True

    def test_has_thinking_content_false(self) -> None:
        state = _make_state()
        assert _CV["has_thinking_content"].fget(state) is False

    def test_has_thinking_content_true(self) -> None:
        state = _make_state()
        state.thinking_items = [_thinking()]
        assert _CV["has_thinking_content"].fget(state) is True

    def test_get_unique_reasoning_sessions(self) -> None:
        state = _make_state()
        state.thinking_items = [
            _thinking(ThinkingType.REASONING, "r1"),
            _thinking(ThinkingType.TOOL_CALL, "tc1"),
            _thinking(ThinkingType.REASONING, "r2"),
        ]
        result = _CV["get_unique_reasoning_sessions"].fget(state)
        assert result == ["r1", "r2"]

    def test_get_unique_tool_calls(self) -> None:
        state = _make_state()
        state.thinking_items = [
            _thinking(ThinkingType.REASONING, "r1"),
            _thinking(ThinkingType.TOOL_CALL, "tc1"),
            _thinking(ThinkingType.TOOL_CALL, "tc2"),
        ]
        result = _CV["get_unique_tool_calls"].fget(state)
        assert result == ["tc1", "tc2"]

    def test_get_unique_reasoning_sessions_empty(self) -> None:
        state = _make_state()
        assert _CV["get_unique_reasoning_sessions"].fget(state) == []

    def test_get_unique_tool_calls_empty(self) -> None:
        state = _make_state()
        assert _CV["get_unique_tool_calls"].fget(state) == []

    def test_get_last_assistant_message_text(self) -> None:
        state = _make_state()
        state.messages = [
            _message(MessageType.HUMAN, "hi"),
            _message(MessageType.ASSISTANT, "hello"),
            _message(MessageType.HUMAN, "bye"),
        ]
        result = _CV["get_last_assistant_message_text"].fget(state)
        assert result == "hello"

    def test_get_last_assistant_message_text_empty(self) -> None:
        state = _make_state()
        result = _CV["get_last_assistant_message_text"].fget(state)
        assert result == ""

    def test_get_last_assistant_message_picks_last(self) -> None:
        state = _make_state()
        state.messages = [
            _message(MessageType.ASSISTANT, "first"),
            _message(MessageType.ASSISTANT, "second"),
        ]
        result = _CV["get_last_assistant_message_text"].fget(state)
        assert result == "second"


# ============================================================================
# _reset_ui_state
# ============================================================================


class TestResetUIState:
    def test_clears_all(self) -> None:
        state = _make_state()
        state.messages = [_message()]
        state.thinking_items = [_thinking()]
        state.image_chunks = [MagicMock()]
        state.prompt = "some prompt"
        state.show_thinking = True
        state.available_commands = [MagicMock()]

        state._reset_ui_state()

        assert state.messages == []
        assert state.thinking_items == []
        assert state.image_chunks == []
        assert state.prompt == ""
        assert state.show_thinking is False
        assert state.available_commands == []


# ============================================================================
# _load_config
# ============================================================================


class TestLoadConfig:
    def test_loads_from_registry(self) -> None:
        state = _make_state()
        fn = _unwrap("_load_config")

        mock_config = MagicMock()
        mock_config.file_upload.max_file_size_mb = 25
        mock_config.file_upload.max_files_per_thread = 5

        with patch(f"{_PATCH}.service_registry") as mock_reg:
            mock_reg.return_value.get.return_value = mock_config
            fn(state)

        assert state.max_file_size_mb == 25
        assert state.max_files_per_thread == 5

    def test_no_config_keeps_defaults(self) -> None:
        state = _make_state()
        fn = _unwrap("_load_config")

        with patch(f"{_PATCH}.service_registry") as mock_reg:
            mock_reg.return_value.get.return_value = None
            fn(state)

        assert state.max_file_size_mb == 50
        assert state.max_files_per_thread == 10


# ============================================================================
# initialize
# ============================================================================


class TestInitialize:
    @pytest.mark.asyncio
    async def test_first_init(self) -> None:
        state = _make_state()
        fn = _unwrap("initialize")

        mock_svc = MagicMock()
        mock_svc.create_new_thread.return_value = _thread_model("new-t")
        state._mock_thread_service = mock_svc

        with patch(f"{_PATCH}.service_registry") as mock_reg:
            mock_reg.return_value.get.return_value = None
            await fn(state)

        assert state._initialized is True
        assert state._current_user_id == "42"
        assert state._thread.thread_id == "new-t"

    @pytest.mark.asyncio
    async def test_already_initialized_same_user(self) -> None:
        state = _make_state()
        state._initialized = True
        state._current_user_id = "42"

        fn = _unwrap("initialize")
        await fn(state)

        # Should not reinitialize
        assert state._initialized is True

    @pytest.mark.asyncio
    async def test_different_user_reinitializes(self) -> None:
        state = _make_state(user_id="99")
        state._initialized = True
        state._current_user_id = "42"  # Different from "99"

        fn = _unwrap("initialize")

        mock_svc = MagicMock()
        mock_svc.create_new_thread.return_value = _thread_model("new-t")
        state._mock_thread_service = mock_svc

        with patch(f"{_PATCH}.service_registry") as mock_reg:
            mock_reg.return_value.get.return_value = None
            await fn(state)

        assert state._current_user_id == "99"
        assert state._initialized is True

    @pytest.mark.asyncio
    async def test_unauthenticated_user(self) -> None:
        state = _make_state(authenticated=False)
        fn = _unwrap("initialize")

        mock_svc = MagicMock()
        mock_svc.create_new_thread.return_value = _thread_model("empty")
        state._mock_thread_service = mock_svc

        with patch(f"{_PATCH}.service_registry") as mock_reg:
            mock_reg.return_value.get.return_value = None
            await fn(state)

        assert state._initialized is True
        assert state._current_user_id == ""

    @pytest.mark.asyncio
    async def test_load_user_prompts_error_handled(self) -> None:
        state = _make_state()
        fn = _unwrap("initialize")

        mock_svc = MagicMock()
        mock_svc.create_new_thread.return_value = _thread_model("new-t")
        state._mock_thread_service = mock_svc

        async def _fail_prompts(user_id: int) -> None:  # noqa: ARG001
            raise RuntimeError("prompt load failed")

        state._load_user_prompts_as_commands = _fail_prompts

        with patch(f"{_PATCH}.service_registry") as mock_reg:
            mock_reg.return_value.get.return_value = None
            # Should not raise, error is caught
            await fn(state)

        assert state._initialized is True


# ============================================================================
# new_thread
# ============================================================================


class TestNewThread:
    @pytest.mark.asyncio
    async def test_creates_new_thread(self) -> None:
        state = _make_state()
        state._initialized = True
        state._thread = _thread_model("old", state=ThreadStatus.ACTIVE)
        state.messages = [_message()]

        fn = _unwrap("new_thread")

        mock_svc = MagicMock()
        mock_svc.create_new_thread.return_value = _thread_model("new-t")
        state._mock_thread_service = mock_svc

        await fn(state)

        assert state._thread.thread_id == "new-t"
        assert state.messages == []
        assert state.thinking_items == []
        assert state.prompt == ""

    @pytest.mark.asyncio
    async def test_skips_if_already_empty(self) -> None:
        state = _make_state()
        state._initialized = True
        state._thread = _thread_model("current", state=ThreadStatus.NEW)
        state.messages = []

        fn = _unwrap("new_thread")
        await fn(state)

        # Thread ID unchanged
        assert state._thread.thread_id == "current"

    @pytest.mark.asyncio
    async def test_calls_initialize_if_not_initialized(self) -> None:
        state = _make_state()
        state._initialized = False

        fn = _unwrap("new_thread")

        # Mock initialize to set _initialized
        async def _mock_init(self_):
            self_._initialized = True
            self_._thread = _thread_model("init-t", state=ThreadStatus.NEW)

        with patch.object(type(state), "initialize", _mock_init):
            await fn(state)

        # Since thread is NEW with no messages, it should skip creating another
        assert state._initialized is True


# ============================================================================
# set_thread
# ============================================================================


class TestSetThread:
    def test_sets_thread_data(self) -> None:
        state = _make_state()
        fn = _unwrap("set_thread")

        msgs = [_message(MessageType.ASSISTANT, "hi")]
        thread = _thread_model("t2", ai_model="claude-3", messages=msgs)

        fn(state, thread)

        assert state._thread.thread_id == "t2"
        assert state.messages == msgs
        assert state.selected_model == "claude-3"
        assert state.thinking_items == []
        assert state.prompt == ""


# ============================================================================
# set_prompt
# ============================================================================


class TestSetPrompt:
    def test_sets_prompt(self) -> None:
        state = _make_state()
        fn = _unwrap("set_prompt")

        fn(state, "hello world")

        assert state.prompt == "hello world"

    def test_empty_prompt(self) -> None:
        state = _make_state()
        fn = _unwrap("set_prompt")

        fn(state, "")

        assert state.prompt == ""


# ============================================================================
# set_suggestions
# ============================================================================


class TestSetSuggestions:
    def test_dict_input(self) -> None:
        state = _make_state()
        fn = _unwrap("set_suggestions")

        fn(state, [{"prompt": "test prompt", "icon": "star"}])

        assert len(state.suggestions) == 1
        assert state.suggestions[0].prompt == "test prompt"

    def test_model_input(self) -> None:
        state = _make_state()
        fn = _unwrap("set_suggestions")

        s = Suggestion(prompt="p1")
        fn(state, [s])

        assert len(state.suggestions) == 1
        assert state.suggestions[0] is s

    def test_empty_list(self) -> None:
        state = _make_state()
        fn = _unwrap("set_suggestions")

        fn(state, [])

        assert state.suggestions == []


# ============================================================================
# set_with_thread_list
# ============================================================================


class TestSetWithThreadList:
    def test_sets_flag(self) -> None:
        state = _make_state()
        fn = _unwrap("set_with_thread_list")

        fn(state, True)
        assert state.with_thread_list is True

        fn(state, False)
        assert state.with_thread_list is False


# ============================================================================
# toggle_thinking_expanded
# ============================================================================


class TestToggleThinkingExpanded:
    def test_toggles(self) -> None:
        state = _make_state()
        fn = _unwrap("toggle_thinking_expanded")

        assert state.thinking_expanded is False
        fn(state)
        assert state.thinking_expanded is True
        fn(state)
        assert state.thinking_expanded is False


# ============================================================================
# toggle_web_search
# ============================================================================


class TestToggleWebSearch:
    def test_toggles(self) -> None:
        state = _make_state()
        fn = _unwrap("toggle_web_search")

        assert state.web_search_enabled is False
        fn(state)
        assert state.web_search_enabled is True
        fn(state)
        assert state.web_search_enabled is False


# ============================================================================
# clear
# ============================================================================


class TestClear:
    def test_clears_thread(self) -> None:
        state = _make_state()
        state._thread = _thread_model(
            "t1",
            state=ThreadStatus.ACTIVE,
            messages=[_message()],
        )
        state.messages = [_message()]
        state.prompt = "test"
        state.thinking_items = [_thinking()]
        state.image_chunks = [MagicMock()]
        state.show_thinking = True
        state.uploaded_files = [MagicMock()]

        fn = _unwrap("clear")

        with patch(f"{_PATCH}.ModelManager") as mock_mm:
            mock_mm.return_value.get_default_model.return_value = "gpt-5-mini"
            fn(state)

        assert state._thread.messages == []
        assert state._thread.state == ThreadStatus.NEW
        assert state._thread.ai_model == "gpt-5-mini"
        assert state._thread.active is True
        assert state._thread.prompt == ""
        assert state._thread.mcp_server_ids == []
        assert state.prompt == ""
        assert state.messages == []
        assert state.thinking_items == []
        assert state.image_chunks == []
        assert state.show_thinking is False
        assert state.uploaded_files == []


# ============================================================================
# persist_current_thread (background task)
# ============================================================================


class TestPersistCurrentThread:
    @pytest.mark.asyncio
    async def test_new_thread_skips(self) -> None:
        state = _make_state()
        state._thread = _thread_model(state=ThreadStatus.NEW)

        fn = _unwrap("persist_current_thread")
        [c async for c in fn(state)]

        # No save should happen

    @pytest.mark.asyncio
    async def test_no_user_skips(self) -> None:
        state = _make_state(authenticated=False)
        state._thread = _thread_model(state=ThreadStatus.ACTIVE)

        fn = _unwrap("persist_current_thread")
        [c async for c in fn(state)]

        # No save should happen

    @pytest.mark.asyncio
    async def test_saves_thread(self) -> None:
        state = _make_state()
        state._thread = _thread_model(state=ThreadStatus.ACTIVE)

        mock_svc = MagicMock()
        mock_svc.save_thread = AsyncMock()
        state._mock_thread_service = mock_svc

        fn = _unwrap("persist_current_thread")
        [c async for c in fn(state)]

        mock_svc.save_thread.assert_awaited_once()

    @pytest.mark.asyncio
    async def test_save_error_handled(self) -> None:
        state = _make_state()
        state._thread = _thread_model(state=ThreadStatus.ACTIVE)

        mock_svc = MagicMock()
        mock_svc.save_thread = AsyncMock(side_effect=RuntimeError("DB error"))
        state._mock_thread_service = mock_svc

        fn = _unwrap("persist_current_thread")
        # Should not raise
        [c async for c in fn(state)]


# ============================================================================
# load_thread (background task)
# ============================================================================


class TestLoadThread:
    @pytest.mark.asyncio
    async def test_unauthenticated_returns(self) -> None:
        state = _make_state(authenticated=False)
        fn = _unwrap("load_thread")

        [c async for c in fn(state, "t1")]

        # No crash, just returns

    @pytest.mark.asyncio
    async def test_thread_not_found(self) -> None:
        state = _make_state()
        fn = _unwrap("load_thread")

        mock_svc = MagicMock()
        mock_svc.load_thread = AsyncMock(return_value=None)
        state._mock_thread_service = mock_svc

        [c async for c in fn(state, "t1")]

        # Thread not loaded, no crash

    @pytest.mark.asyncio
    async def test_successful_load(self) -> None:
        state = _make_state()
        fn = _unwrap("load_thread")

        msgs = [_message(MessageType.ASSISTANT, "Hello")]
        thread = _thread_model(
            "t2",
            state=ThreadStatus.ACTIVE,
            ai_model="claude-3",
            messages=msgs,
        )

        mock_svc = MagicMock()
        mock_svc.load_thread = AsyncMock(return_value=thread)
        state._mock_thread_service = mock_svc

        with patch(f"{_PATCH}.ModelManager") as mock_mm:
            model = MagicMock(supports_tools=True, supports_skills=True)
            mock_mm.return_value.get_model.return_value = model

            [c async for c in fn(state, "t2")]

        assert state._thread.thread_id == "t2"
        assert state.selected_model == "claude-3"
        assert state.messages[0].done is True

    @pytest.mark.asyncio
    async def test_load_error_handled(self) -> None:
        state = _make_state()
        fn = _unwrap("load_thread")

        mock_svc = MagicMock()
        mock_svc.load_thread = AsyncMock(side_effect=RuntimeError("DB error"))
        state._mock_thread_service = mock_svc

        # Should not raise
        [c async for c in fn(state, "t1")]

    @pytest.mark.asyncio
    async def test_load_model_no_tools(self) -> None:
        state = _make_state()
        fn = _unwrap("load_thread")

        thread = _thread_model(
            "t3",
            state=ThreadStatus.ACTIVE,
            ai_model="simple-model",
            mcp_server_ids=[1, 2],
            skill_openai_ids=["sk1"],
        )

        mock_svc = MagicMock()
        mock_svc.load_thread = AsyncMock(return_value=thread)
        state._mock_thread_service = mock_svc

        with patch(f"{_PATCH}.ModelManager") as mock_mm:
            model = MagicMock(supports_tools=False, supports_skills=False)
            mock_mm.return_value.get_model.return_value = model

            [c async for c in fn(state, "t3")]

        # MCP and skills should be cleared
        assert state.selected_mcp_servers == []
        assert state.selected_skills == []

    @pytest.mark.asyncio
    async def test_load_model_none(self) -> None:
        """Model not found in ModelManager."""
        state = _make_state()
        fn = _unwrap("load_thread")

        thread = _thread_model(
            "t4",
            state=ThreadStatus.ACTIVE,
            ai_model="unknown-model",
        )

        mock_svc = MagicMock()
        mock_svc.load_thread = AsyncMock(return_value=thread)
        state._mock_thread_service = mock_svc

        with patch(f"{_PATCH}.ModelManager") as mock_mm:
            mock_mm.return_value.get_model.return_value = None

            [c async for c in fn(state, "t4")]

        assert state.selected_mcp_servers == []
        assert state.selected_skills == []


# ============================================================================
# _stop_loading_state
# ============================================================================


class TestStopLoadingState:
    @pytest.mark.asyncio
    async def test_clears_loading(self) -> None:
        state = _make_state()
        fn = _unwrap("_stop_loading_state")

        await fn(state)
        # Just verifies no crash — actual assertion needs
        # get_state(ThreadListState) mock which our stub provides



"""Tests for FileUploadService.

Covers file validation, retry logic, vector store lifecycle,
file deletion cascade, and error handling.
"""

from pathlib import Path
from types import SimpleNamespace
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from appkit_assistant.backend.services.file_upload_service import (
    FileUploadError,
    FileUploadService,
)
from appkit_assistant.configuration import FileUploadConfig

_PATCH = "appkit_assistant.backend.services.file_upload_service"


def _mock_session(**overrides: object) -> MagicMock:
    """Create a mock session with sync add() and async commit/flush/refresh."""
    s = MagicMock()
    s.commit = AsyncMock()
    s.flush = AsyncMock()
    s.refresh = AsyncMock()
    s.execute = AsyncMock()
    s.expunge_all = MagicMock()
    for k, v in overrides.items():
        setattr(s, k, v)
    return s


def _db_context(session: MagicMock | None = None):
    s = session or AsyncMock()
    cm = AsyncMock()
    cm.__aenter__ = AsyncMock(return_value=s)
    cm.__aexit__ = AsyncMock(return_value=False)
    return cm


@pytest.fixture
def mock_client() -> MagicMock:
    client = MagicMock()
    client.files = MagicMock()
    client.files.create = AsyncMock()
    client.files.delete = AsyncMock()
    client.vector_stores = MagicMock()
    client.vector_stores.create = AsyncMock()
    client.vector_stores.retrieve = AsyncMock()
    client.vector_stores.delete = AsyncMock()
    client.vector_stores.files = MagicMock()
    client.vector_stores.files.create = AsyncMock()
    client.vector_stores.files.list = AsyncMock()
    client.vector_stores.files.delete = AsyncMock()
    return client


@pytest.fixture
def config() -> FileUploadConfig:
    return FileUploadConfig(
        max_file_size_mb=10,
        max_files_per_thread=5,
        vector_store_expiration_days=7,
    )


@pytest.fixture
def service(mock_client: MagicMock, config: FileUploadConfig) -> FileUploadService:
    return FileUploadService(client=mock_client, config=config)


# ============================================================================
# Initialisation
# ============================================================================


class TestInit:
    def test_default_config(self, mock_client: MagicMock) -> None:
        svc = FileUploadService(client=mock_client)
        assert svc.config is not None
        assert svc.client is mock_client

    def test_custom_config(
        self, mock_client: MagicMock, config: FileUploadConfig
    ) -> None:
        svc = FileUploadService(client=mock_client, config=config)
        assert svc._max_file_size_bytes == 10 * 1024 * 1024  # noqa: SLF001


# ============================================================================
# upload_file — validation
# ============================================================================


class TestUploadFileValidation:
    @pytest.mark.asyncio
    async def test_file_not_found(self, service: FileUploadService) -> None:
        with pytest.raises(FileUploadError, match="nicht gefunden"):
            await service.upload_file("/nonexistent/file.txt", 1, 1)

    @pytest.mark.asyncio
    async def test_file_too_large(
        self, service: FileUploadService, tmp_path: Path
    ) -> None:
        big_file = tmp_path / "big.pdf"
        big_file.write_bytes(b"x" * (11 * 1024 * 1024))  # 11MB > 10MB

        with pytest.raises(FileUploadError, match="maximale Größe"):
            await service.upload_file(str(big_file), 1, 1)

    @pytest.mark.asyncio
    async def test_max_files_exceeded(
        self, service: FileUploadService, tmp_path: Path
    ) -> None:
        small_file = tmp_path / "ok.txt"
        small_file.write_text("content")

        mock_result = MagicMock()
        mock_result.scalars.return_value.all.return_value = [MagicMock()] * 5

        mock_session = _mock_session()
        mock_session.execute = AsyncMock(return_value=mock_result)

        with patch(
            "appkit_assistant.backend.services.file_upload_service.get_asyncdb_session"
        ) as mock_ctx:
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)

            with pytest.raises(FileUploadError, match="Maximum files"):
                await service.upload_file(str(small_file), 1, 1)

    @pytest.mark.asyncio
    async def test_successful_upload(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
        tmp_path: Path,
    ) -> None:
        small_file = tmp_path / "doc.txt"
        small_file.write_text("content")

        mock_client.files.create.return_value = MagicMock(id="file-123")

        mock_result = MagicMock()
        mock_result.scalars.return_value.all.return_value = []
        mock_session = _mock_session()
        mock_session.execute = AsyncMock(return_value=mock_result)

        with patch(
            "appkit_assistant.backend.services.file_upload_service.get_asyncdb_session"
        ) as mock_ctx:
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)

            result = await service.upload_file(str(small_file), 1, 1)
        assert result == "file-123"


# ============================================================================
# _upload_with_retry
# ============================================================================


class TestUploadWithRetry:
    @pytest.mark.asyncio
    async def test_success_first_attempt(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
        tmp_path: Path,
    ) -> None:
        f = tmp_path / "test.txt"
        f.write_text("data")
        mock_client.files.create.return_value = MagicMock(id="file-ok")

        result = await service._upload_with_retry(f)  # noqa: SLF001
        assert result == "file-ok"

    @pytest.mark.asyncio
    async def test_retry_on_failure(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
        tmp_path: Path,
    ) -> None:
        f = tmp_path / "test.txt"
        f.write_text("data")
        mock_client.files.create.side_effect = [
            Exception("timeout"),
            MagicMock(id="file-ok"),
        ]

        with patch("asyncio.sleep", new_callable=AsyncMock):
            result = await service._upload_with_retry(f)  # noqa: SLF001
        assert result == "file-ok"

    @pytest.mark.asyncio
    async def test_all_retries_fail(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
        tmp_path: Path,
    ) -> None:
        f = tmp_path / "test.txt"
        f.write_text("data")
        mock_client.files.create.side_effect = Exception("always fails")

        with (
            patch("asyncio.sleep", new_callable=AsyncMock),
            pytest.raises(FileUploadError, match="Failed to upload"),
        ):
            await service._upload_with_retry(f)  # noqa: SLF001


# ============================================================================
# _create_vector_store_with_retry
# ============================================================================


class TestCreateVectorStoreRetry:
    @pytest.mark.asyncio
    async def test_success(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        mock_client.vector_stores.create.return_value = MagicMock(
            id="vs-123", name="Thread-abc"
        )

        result = await service._create_vector_store_with_retry("abc")  # noqa: SLF001
        assert result.id == "vs-123"

    @pytest.mark.asyncio
    async def test_all_retries_fail(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        mock_client.vector_stores.create.side_effect = Exception("fail")

        with (
            patch("asyncio.sleep", new_callable=AsyncMock),
            pytest.raises(FileUploadError, match="Failed to create vector store"),
        ):
            await service._create_vector_store_with_retry("abc")  # noqa: SLF001


# ============================================================================
# get_vector_store
# ============================================================================


class TestGetVectorStore:
    @pytest.mark.asyncio
    async def test_creates_new_when_no_existing(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        mock_thread = MagicMock(vector_store_id=None)
        mock_result = MagicMock()
        mock_result.scalar_one_or_none.return_value = mock_thread
        mock_session = _mock_session()
        mock_session.execute = AsyncMock(return_value=mock_result)

        mock_client.vector_stores.create.return_value = SimpleNamespace(
            id="vs-new", name="Thread-uuid1"
        )

        with patch(
            "appkit_assistant.backend.services.file_upload_service.get_asyncdb_session"
        ) as mock_ctx:
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)

            vs_id, vs_name = await service.get_vector_store(1, "uuid1")

        assert vs_id == "vs-new"
        assert "uuid1" in vs_name

    @pytest.mark.asyncio
    async def test_returns_existing_valid(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        mock_thread = MagicMock(vector_store_id="vs-existing")
        mock_result = MagicMock()
        mock_result.scalar_one_or_none.return_value = mock_thread
        mock_session = _mock_session()
        mock_session.execute = AsyncMock(return_value=mock_result)

        mock_client.vector_stores.retrieve.return_value = MagicMock(name="Thread-uuid1")

        with patch(
            "appkit_assistant.backend.services.file_upload_service.get_asyncdb_session"
        ) as mock_ctx:
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)

            vs_id, _vs_name = await service.get_vector_store(1, "uuid1")

        assert vs_id == "vs-existing"

    @pytest.mark.asyncio
    async def test_thread_not_found_raises(self, service: FileUploadService) -> None:
        mock_result = MagicMock()
        mock_result.scalar_one_or_none.return_value = None
        mock_session = _mock_session()
        mock_session.execute = AsyncMock(return_value=mock_result)

        with patch(
            "appkit_assistant.backend.services.file_upload_service.get_asyncdb_session"
        ) as mock_ctx:
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)

            with pytest.raises(FileUploadError, match="Thread not found"):
                await service.get_vector_store(999, "uuid1")


# ============================================================================
# delete_files — three-level cascade
# ============================================================================


class TestDeleteFiles:
    @pytest.mark.asyncio
    async def test_empty_list(self, service: FileUploadService) -> None:
        result = await service.delete_files([])
        assert result == {}

    @pytest.mark.asyncio
    async def test_full_cascade(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        db_file = MagicMock(openai_file_id="file-1", vector_store_id="vs-1")
        mock_result = MagicMock()
        mock_result.scalars.return_value.all.return_value = [db_file]
        mock_session = _mock_session()
        mock_session.execute = AsyncMock(return_value=mock_result)

        with patch(
            "appkit_assistant.backend.services.file_upload_service.get_asyncdb_session"
        ) as mock_ctx:
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)

            result = await service.delete_files(["file-1"])

        assert result["file-1"] is True
        # Vector store file delete was called
        mock_client.vector_stores.files.delete.assert_awaited()
        # OpenAI file delete was called
        mock_client.files.delete.assert_awaited()


# ============================================================================
# _delete_files_from_openai
# ============================================================================


class TestDeleteFilesFromOpenai:
    @pytest.mark.asyncio
    async def test_success(
        self,
        service: FileUploadService,
    ) -> None:
        result = await service._delete_files_from_openai(["f1", "f2"])  # noqa: SLF001
        assert result == {"f1": True, "f2": True}

    @pytest.mark.asyncio
    async def test_partial_failure(
        self,
        service: FileUploadService,
    ) -> None:
        service.client.files.delete.side_effect = [None, Exception("fail")]
        result = await service._delete_files_from_openai(["f1", "f2"])  # noqa: SLF001
        assert result["f1"] is True
        assert result["f2"] is False


# ============================================================================
# _wait_for_processing — progress chunks
# ============================================================================


class TestWaitForProcessing:
    @pytest.mark.asyncio
    async def test_all_completed(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        vs_file = MagicMock(id="f1", status="completed")
        mock_client.vector_stores.files.list.return_value = MagicMock(data=[vs_file])

        chunks = [
            chunk
            async for chunk in service._wait_for_processing(  # noqa: SLF001
                "vs-1", ["f1"], ["doc.pdf"]
            )
        ]

        statuses = [c.chunk_metadata.get("status") for c in chunks]
        assert "indexing" in statuses
        assert "completed" in statuses

    @pytest.mark.asyncio
    async def test_file_failed(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        vs_file = MagicMock(
            id="f1", status="failed", last_error=MagicMock(message="bad format")
        )
        mock_client.vector_stores.files.list.return_value = MagicMock(data=[vs_file])

        chunks = [
            chunk
            async for chunk in service._wait_for_processing(  # noqa: SLF001
                "vs-1", ["f1"], ["doc.pdf"]
            )
        ]

        statuses = [c.chunk_metadata.get("status") for c in chunks]
        assert "failed" in statuses

    @pytest.mark.asyncio
    async def test_empty_file_ids(self, service: FileUploadService) -> None:
        chunks = [
            chunk
            async for chunk in service._wait_for_processing(  # noqa: SLF001
                "vs-1", [], []
            )
        ]
        assert chunks == []


# ============================================================================
# delete_vector_store
# ============================================================================


class TestDeleteVectorStore:
    @pytest.mark.asyncio
    async def test_empty_id(self, service: FileUploadService) -> None:
        result = await service.delete_vector_store("")
        assert result["deleted"] is False

    @pytest.mark.asyncio
    async def test_successful_deletion(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        mock_session = _mock_session()
        mock_result = MagicMock()
        mock_result.scalars.return_value.all.return_value = []
        mock_session.execute = AsyncMock(return_value=mock_result)

        mock_client.vector_stores.files.list.return_value = MagicMock(data=[])

        with (
            patch(
                "appkit_assistant.backend.services.file_upload_service."
                "get_asyncdb_session"
            ) as mock_ctx,
            patch(
                "appkit_assistant.backend.services.file_upload_service.file_upload_repo"
            ) as mock_repo,
        ):
            mock_ctx.return_value.__aenter__ = AsyncMock(return_value=mock_session)
            mock_ctx.return_value.__aexit__ = AsyncMock(return_value=False)
            mock_repo.find_by_vector_store = AsyncMock(return_value=[])

            result = await service.delete_vector_store("vs-1")

        assert result["deleted"] is True
        mock_client.vector_stores.delete.assert_awaited_once()


# ============================================================================
# cleanup_deleted_thread
# ============================================================================


class TestCleanupDeletedThread:
    @pytest.mark.asyncio
    async def test_no_vector_store(self, service: FileUploadService) -> None:
        result = await service.cleanup_deleted_thread(1, None)
        assert result["vector_store_deleted"] is False
        assert result["errors"] == []

    @pytest.mark.asyncio
    async def test_with_vector_store(self, service: FileUploadService) -> None:
        with patch.object(
            service,
            "delete_vector_store",
            new_callable=AsyncMock,
            return_value={"deleted": True, "files_found": 2, "files_deleted": 2},
        ):
            result = await service.cleanup_deleted_thread(1, "vs-1")
        assert result["vector_store_deleted"] is True

    @pytest.mark.asyncio
    async def test_vector_store_deletion_failure(
        self, service: FileUploadService
    ) -> None:
        with patch.object(
            service,
            "delete_vector_store",
            new_callable=AsyncMock,
            return_value={"deleted": False, "files_found": 0, "files_deleted": 0},
        ):
            result = await service.cleanup_deleted_thread(1, "vs-1")
        assert result["vector_store_deleted"] is False
        assert len(result["errors"]) == 1


# ============================================================================
# Supplementary tests for uncovered code paths
# ============================================================================


class TestRecreateVectorStore:
    @pytest.mark.asyncio
    async def test_recreate_migrates_files(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        """_recreate_vector_store creates new VS and migrates files."""
        session = _mock_session()
        thread = MagicMock()
        thread.vector_store_id = "old-vs"

        file1 = MagicMock()
        file1.openai_file_id = "file-1"
        file2 = MagicMock()
        file2.openai_file_id = "file-2"

        new_vs = MagicMock()
        new_vs.id = "new-vs"
        new_vs.name = "Thread-uuid"

        with patch(f"{_PATCH}.file_upload_repo") as fr:
            fr.find_by_thread = AsyncMock(return_value=[file1, file2])
            mock_client.vector_stores.create = AsyncMock(return_value=new_vs)
            mock_client.vector_stores.files.create = AsyncMock()

            vs_id, _vs_name = await service._recreate_vector_store(
                session, thread, "uuid"
            )

        assert vs_id == "new-vs"
        assert thread.vector_store_id == "new-vs"
        assert mock_client.vector_stores.files.create.call_count == 2
        session.commit.assert_awaited_once()

    @pytest.mark.asyncio
    async def test_recreate_handles_file_add_failure(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        """_recreate_vector_store continues when one file fails to add."""
        session = _mock_session()
        thread = MagicMock()
        thread.vector_store_id = "old-vs"

        file1 = MagicMock()
        file1.openai_file_id = "file-ok"
        file2 = MagicMock()
        file2.openai_file_id = "file-fail"

        new_vs = MagicMock()
        new_vs.id = "new-vs"
        new_vs.name = "Thread-uuid"

        with patch(f"{_PATCH}.file_upload_repo") as fr:
            fr.find_by_thread = AsyncMock(return_value=[file1, file2])
            mock_client.vector_stores.create = AsyncMock(return_value=new_vs)
            mock_client.vector_stores.files.create = AsyncMock(
                side_effect=[None, RuntimeError("api error")]
            )

            vs_id, _ = await service._recreate_vector_store(session, thread, "uuid")

        assert vs_id == "new-vs"


class TestAddFilesToVectorStore:
    @pytest.mark.asyncio
    async def test_add_files_success(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        """_add_files_to_vector_store adds files and writes DB records."""
        mock_client.vector_stores.files.create = AsyncMock()
        session = _mock_session()

        with patch(
            f"{_PATCH}.get_asyncdb_session",
            return_value=_db_context(session),
        ):
            await service._add_files_to_vector_store(
                vector_store_id="vs-1",
                vector_store_name="test",
                file_ids=["f1", "f2"],
                thread_id=1,
                user_id=1,
                filenames=["a.txt", "b.txt"],
                file_sizes=[100, 200],
                ai_model="gpt-4",
            )

        assert mock_client.vector_stores.files.create.call_count == 2
        session.commit.assert_awaited_once()

    @pytest.mark.asyncio
    async def test_add_files_empty(
        self,
        service: FileUploadService,
    ) -> None:
        """_add_files_to_vector_store returns early for empty list."""
        await service._add_files_to_vector_store(
            vector_store_id="vs-1",
            vector_store_name="test",
            file_ids=[],
            thread_id=1,
            user_id=1,
            filenames=[],
            file_sizes=[],
        )

    @pytest.mark.asyncio
    async def test_add_files_api_error(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        """_add_files_to_vector_store raises FileUploadError on API failure."""
        mock_client.vector_stores.files.create = AsyncMock(
            side_effect=RuntimeError("api")
        )
        with pytest.raises(FileUploadError, match="Failed to add file"):
            await service._add_files_to_vector_store(
                vector_store_id="vs-1",
                vector_store_name="test",
                file_ids=["f1"],
                thread_id=1,
                user_id=1,
                filenames=["a.txt"],
                file_sizes=[100],
            )


class TestProcessFiles:
    @pytest.mark.asyncio
    async def test_empty_file_paths(
        self,
        service: FileUploadService,
    ) -> None:
        """process_files returns immediately for empty list."""
        chunks = [c async for c in service.process_files([], 1, "u", 1)]
        assert chunks == []

    @pytest.mark.asyncio
    async def test_full_flow(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
        tmp_path: Path,
    ) -> None:
        """process_files uploads, creates VS, adds, tracks, and waits."""
        f = tmp_path / "test.txt"
        f.write_text("hello")

        mock_client.vector_stores.files.create = AsyncMock()

        vs_file = SimpleNamespace(id="file-1", status="completed", last_error=None)
        mock_client.vector_stores.files.list = AsyncMock(
            return_value=SimpleNamespace(data=[vs_file])
        )

        session = _mock_session()
        session.add = MagicMock()

        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(session),
            ),
            patch.object(
                service,
                "upload_file",
                new_callable=AsyncMock,
                return_value="file-1",
            ),
            patch.object(
                service,
                "get_vector_store",
                new_callable=AsyncMock,
                return_value=("vs-new", "Thread-u"),
            ),
        ):
            chunks = [
                c async for c in service.process_files([str(f)], 1, "u", 1, "gpt-4")
            ]

        assert len(chunks) >= 3
        statuses = [c.chunk_metadata.get("status") for c in chunks if c.chunk_metadata]
        assert "uploading" in statuses

    @pytest.mark.asyncio
    async def test_cleanup_on_failure(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
        tmp_path: Path,
    ) -> None:
        """process_files cleans up uploaded files on failure."""
        f = tmp_path / "test.txt"
        f.write_text("hello")

        mock_client.files.create = AsyncMock(return_value=SimpleNamespace(id="file-1"))

        session = _mock_session()
        session.add = MagicMock()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(session),
            ),
            patch.object(
                service,
                "upload_file",
                new_callable=AsyncMock,
                return_value="file-1",
            ),
            patch.object(
                service,
                "get_vector_store",
                new_callable=AsyncMock,
                side_effect=RuntimeError("vs fail"),
            ),
            patch.object(
                service,
                "delete_files",
                new_callable=AsyncMock,
            ) as mock_del,
        ):
            with pytest.raises(RuntimeError, match="vs fail"):
                async for _ in service.process_files([str(f)], 1, "u", 1):
                    pass
            mock_del.assert_awaited_once()


class TestGetVectorStoreRecreate:
    @staticmethod
    def _mock_session_with_thread(thread: MagicMock) -> AsyncMock:
        session = _mock_session()
        result = MagicMock()
        result.scalar_one_or_none.return_value = thread
        session.execute = AsyncMock(return_value=result)
        return session

    @pytest.mark.asyncio
    async def test_existing_vs_not_found_triggers_recreate(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        """get_vector_store recreates VS when 404 from OpenAI."""
        thread = MagicMock()
        thread.vector_store_id = "old-vs"

        session = self._mock_session_with_thread(thread)
        mock_client.vector_stores.retrieve = AsyncMock(
            side_effect=Exception("not found 404")
        )

        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(session),
            ),
            patch.object(
                service,
                "_recreate_vector_store",
                new_callable=AsyncMock,
                return_value=("new-vs", "Thread-u"),
            ) as recreate,
        ):
            vs_id, _vs_name = await service.get_vector_store(1, "u")

        recreate.assert_awaited_once()
        assert vs_id == "new-vs"

    @pytest.mark.asyncio
    async def test_existing_vs_other_error(
        self,
        service: FileUploadService,
        mock_client: MagicMock,
    ) -> None:
        """get_vector_store returns existing VS ID on non-404 error."""
        thread = MagicMock()
        thread.vector_store_id = "old-vs"

        session = self._mock_session_with_thread(thread)
        mock_client.vector_stores.retrieve = AsyncMock(
            side_effect=Exception("server error")
        )

        with patch(
            f"{_PATCH}.get_asyncdb_session",
            return_value=_db_context(session),
        ):
            vs_id, vs_name = await service.get_vector_store(1, "u")

        assert vs_id == "old-vs"
        assert "Thread-u" in vs_name



import reflex as rx


class KeyboardShortcuts(rx.Component):
    """Keyboard shortcut handler component for the composer.

    Handles:
    - Enter without Shift: Submit form (unless command palette is open)
    - ArrowUp/ArrowDown: Navigate command palette
    - Tab: Move to next command in palette
    - Escape: Dismiss command palette
    - Enter when palette open: Select current command
    """

    library = None  # No external library needed
    tag = None

    def add_imports(self) -> dict:
        return {
            "react": [rx.ImportVar(tag="useEffect")],
        }

    def add_hooks(self) -> list[str]:
        return [
            """
useEffect(() => {
    // Helper to scroll selected item into view after DOM update
    const scrollSelectedIntoView = () => {
        // Try multiple times with increasing delays to catch the DOM update
        const attempts = [50, 100, 200];
        attempts.forEach(delay => {
            setTimeout(() => {
                const palette = document.getElementById('command-palette');
                if (!palette) return;

                // Find the selected item by data-selected attribute
                const selected = palette.querySelector('[data-selected="true"]');
                if (selected) {
                    selected.scrollIntoView({ block: 'nearest', behavior: 'smooth' });
                }
            }, delay);
        });
    };

    const handleKeydown = (e) => {
        const textarea = document.getElementById('composer-area');
        const submitBtn = document.getElementById('composer-submit');

        // Only handle if we're focused on the textarea
        if (document.activeElement !== textarea) return;
        if (!submitBtn) return;

        // Check if command palette is open by looking for the palette element
        // Use getBoundingClientRect to check actual visibility (works with
        // absolute positioned elements)
        const paletteEl = document.getElementById('command-palette');
        const paletteOpen = paletteEl !== null
                            && paletteEl.getBoundingClientRect().height > 0;

        // Command palette navigation when open
        if (paletteOpen) {
            if (e.key === 'ArrowUp') {
                e.preventDefault();
                const btn = document.getElementById('cmd-palette-up');
                if (btn) {
                    btn.click();
                    scrollSelectedIntoView();
                }
                return;
            }
            if (e.key === 'ArrowDown' || e.key === 'Tab') {
                e.preventDefault();
                const btn = document.getElementById('cmd-palette-down');
                if (btn) {
                    btn.click();
                    scrollSelectedIntoView();
                }
                return;
            }
            if (e.key === 'Enter' && !e.shiftKey) {
                e.preventDefault();
                const btn = document.getElementById('cmd-palette-select');
                if (btn) btn.click();
                return;
            }
            if (e.key === 'Escape') {
                e.preventDefault();
                const btn = document.getElementById('cmd-palette-dismiss');
                if (btn) btn.click();
                return;
            }
        }

        // Default Enter behavior (submit) when palette is closed
        if (e.key === 'Enter' && !e.shiftKey && !paletteOpen) {
            e.preventDefault();
            if (textarea.value.trim() && !submitBtn.disabled) {
                submitBtn.click();
            }
        }
    };

    // Use document-level listener to ensure we catch events even if useEffect
    // runs before textarea is mounted
    document.addEventListener('keydown', handleKeydown);
    return () => document.removeEventListener('keydown', handleKeydown);
}, []);
            """
        ]


# Create an instance function
keyboard_shortcuts = KeyboardShortcuts.create



# ruff: noqa: ARG002, SLF001, S105, S106, PERF203
"""Tests for MCPServerState.

Covers load, add, modify, delete, toggle, role update,
header parsing, and computed vars.
"""

from __future__ import annotations

from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from appkit_assistant.state.mcp_server_state import MCPServerState

_PATCH = "appkit_assistant.state.mcp_server_state"
_CV = MCPServerState.__dict__


def _unwrap(name: str):
    entry = MCPServerState.__dict__[name]
    return entry.fn if hasattr(entry, "fn") else entry


def _db_context(session: AsyncMock | None = None):
    s = session or AsyncMock()
    cm = AsyncMock()
    cm.__aenter__ = AsyncMock(return_value=s)
    cm.__aexit__ = AsyncMock(return_value=False)
    return cm


def _server(
    server_id: int = 1,
    name: str = "test",
    url: str = "https://mcp.test",
    active: bool = True,
    required_role: str | None = None,
) -> MagicMock:
    s = MagicMock()
    s.id = server_id
    s.name = name
    s.url = url
    s.active = active
    s.required_role = required_role
    s.model_dump.return_value = {
        "id": server_id,
        "name": name,
        "url": url,
        "active": active,
        "required_role": required_role,
    }
    s.model_copy.return_value = MagicMock(
        id=server_id,
        name=name,
        url=url,
        active=not active,
        required_role=required_role,
    )
    return s


class _StubMCPServerState:
    def __init__(self) -> None:
        self.servers: list = []
        self.current_server = None
        self.loading = False
        self.updating_active_server_id: int | None = None
        self.updating_role_server_id: int | None = None
        self.add_modal_open = False
        self.opening_add_modal = False
        self.edit_modal_open = False
        self.opening_edit_server_id: int | None = None
        self.search_filter = ""
        self.available_roles: list = []
        self.role_labels: dict = {}

    load_servers = _unwrap("load_servers")
    load_servers_with_toast = _unwrap("load_servers_with_toast")
    get_server = _unwrap("get_server")
    add_server = _unwrap("add_server")
    modify_server = _unwrap("modify_server")
    delete_server = _unwrap("delete_server")
    toggle_server_active = _unwrap("toggle_server_active")
    update_server_role = _unwrap("update_server_role")
    _parse_headers_from_form = _unwrap("_parse_headers_from_form")
    set_search_filter = _unwrap("set_search_filter")
    open_add_modal = _unwrap("open_add_modal")
    close_add_modal = _unwrap("close_add_modal")
    open_edit_modal = _unwrap("open_edit_modal")
    close_edit_modal = _unwrap("close_edit_modal")
    set_available_roles = _unwrap("set_available_roles")
    set_current_server = _unwrap("set_current_server")


# ================================================================
# Computed vars
# ================================================================


class TestComputedVars:
    def test_server_count(self) -> None:
        state = _StubMCPServerState()
        state.servers = [_server(), _server(2)]
        assert _CV["server_count"].fget(state) == 2

    def test_has_servers_true(self) -> None:
        state = _StubMCPServerState()
        state.servers = [_server()]
        assert _CV["has_servers"].fget(state) is True

    def test_has_servers_false(self) -> None:
        state = _StubMCPServerState()
        state.servers = []
        assert _CV["has_servers"].fget(state) is False

    def test_filtered_servers_no_filter(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.name = "Alpha"
        state.servers = [s]
        state.search_filter = ""
        result = _CV["filtered_servers"].fget(state)
        assert len(result) == 1

    def test_filtered_servers_with_filter(self) -> None:
        state = _StubMCPServerState()
        s1 = MagicMock()
        s1.name = "Alpha"
        s2 = MagicMock()
        s2.name = "Beta"
        state.servers = [s1, s2]
        state.search_filter = "alp"
        result = _CV["filtered_servers"].fget(state)
        assert len(result) == 1
        assert result[0].name == "Alpha"


# ================================================================
# Simple methods
# ================================================================


class TestSimpleMethods:
    def test_set_search_filter(self) -> None:
        state = _StubMCPServerState()
        state.set_search_filter("test")
        assert state.search_filter == "test"

    def test_close_add_modal(self) -> None:
        state = _StubMCPServerState()
        state.add_modal_open = True
        state.close_add_modal()
        assert state.add_modal_open is False

    def test_close_edit_modal(self) -> None:
        state = _StubMCPServerState()
        state.edit_modal_open = True
        state.current_server = _server()
        state.close_edit_modal()
        assert state.edit_modal_open is False
        assert state.current_server is None

    def test_set_available_roles(self) -> None:
        state = _StubMCPServerState()
        roles = [{"value": "admin", "label": "Admin"}]
        labels = {"admin": "Admin"}
        state.set_available_roles(roles, labels)
        assert state.available_roles == roles
        assert state.role_labels == labels

    @pytest.mark.asyncio
    async def test_set_current_server(self) -> None:
        state = _StubMCPServerState()
        s = _server()
        await state.set_current_server(s)
        assert state.current_server is s

    @pytest.mark.asyncio
    async def test_open_add_modal(self) -> None:
        state = _StubMCPServerState()
        [r async for r in state.open_add_modal()]
        assert state.add_modal_open is True

    @pytest.mark.asyncio
    async def test_open_edit_modal_found(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        state.servers = [s]
        [r async for r in state.open_edit_modal(1)]
        assert state.edit_modal_open is True
        assert state.current_server is s


# ================================================================
# Parse headers
# ================================================================


class TestParseHeaders:
    def test_empty(self) -> None:
        state = _StubMCPServerState()
        result = state._parse_headers_from_form({})
        assert result == "{}"

    def test_blank(self) -> None:
        state = _StubMCPServerState()
        result = state._parse_headers_from_form({"headers_json": "  "})
        assert result == "{}"

    def test_valid_json(self) -> None:
        state = _StubMCPServerState()
        result = state._parse_headers_from_form(
            {"headers_json": '{"Authorization": "Bearer x"}'}
        )
        assert "Authorization" in result

    def test_invalid_json(self) -> None:
        state = _StubMCPServerState()
        with pytest.raises(ValueError, match="JSON"):
            state._parse_headers_from_form({"headers_json": "not json"})

    def test_non_dict_json(self) -> None:
        state = _StubMCPServerState()
        with pytest.raises(ValueError, match="dictionary"):
            state._parse_headers_from_form({"headers_json": '["list"]'})

    def test_non_string_values(self) -> None:
        state = _StubMCPServerState()
        with pytest.raises(ValueError, match="Invalid"):
            state._parse_headers_from_form({"headers_json": '{"key": 123}'})


# ================================================================
# Load servers
# ================================================================


class TestLoadServers:
    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _StubMCPServerState()
        s = _server()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_all_ordered_by_name = AsyncMock(return_value=[s])
            await state.load_servers()
        assert len(state.servers) == 1
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_error_raises(self) -> None:
        state = _StubMCPServerState()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                side_effect=RuntimeError("db"),
            ),
            pytest.raises(RuntimeError),
        ):
            await state.load_servers()
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_with_toast_success(self) -> None:
        state = _StubMCPServerState()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_all_ordered_by_name = AsyncMock(return_value=[])
            [r async for r in state.load_servers_with_toast()]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_with_toast_error(self) -> None:
        state = _StubMCPServerState()
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            results = [r async for r in state.load_servers_with_toast()]
        # Should yield toast error
        assert len(results) >= 1


# ================================================================
# Get server
# ================================================================


class TestGetServer:
    @pytest.mark.asyncio
    async def test_found(self) -> None:
        state = _StubMCPServerState()
        s = _server()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=s)
            await state.get_server(1)
        assert state.current_server is not None

    @pytest.mark.asyncio
    async def test_not_found(self) -> None:
        state = _StubMCPServerState()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=None)
            await state.get_server(999)
        assert state.current_server is None

    @pytest.mark.asyncio
    async def test_error(self) -> None:
        state = _StubMCPServerState()
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            await state.get_server(1)
        # Should not raise


# ================================================================
# Add server
# ================================================================


class TestAddServer:
    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _StubMCPServerState()
        saved = _server()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.save = AsyncMock(return_value=saved)
            repo.find_all_ordered_by_name = AsyncMock(return_value=[saved])
            [
                r
                async for r in state.add_server(
                    {"name": "New", "url": "https://new.test"}
                )
            ]
        assert state.add_modal_open is False
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_value_error(self) -> None:
        state = _StubMCPServerState()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.save = AsyncMock(side_effect=ValueError("bad data"))
            [
                r
                async for r in state.add_server(
                    {"name": "X", "url": "x", "headers_json": "{}"}
                )
            ]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_exception(self) -> None:
        state = _StubMCPServerState()
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            [r async for r in state.add_server({"name": "X", "url": "x"})]
        assert state.loading is False


# ================================================================
# Modify server
# ================================================================


class TestModifyServer:
    @pytest.mark.asyncio
    async def test_no_current_server(self) -> None:
        state = _StubMCPServerState()
        state.current_server = None
        [r async for r in state.modify_server({"name": "X", "url": "x"})]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _StubMCPServerState()
        s = _server()
        state.current_server = s
        updated = _server(name="Updated")
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=s)
            repo.save = AsyncMock(return_value=updated)
            repo.find_all_ordered_by_name = AsyncMock(return_value=[updated])
            [
                r
                async for r in state.modify_server(
                    {"name": "Updated", "url": "https://new.test"}
                )
            ]
        assert state.edit_modal_open is False
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_server_not_found(self) -> None:
        state = _StubMCPServerState()
        state.current_server = _server()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=None)
            [r async for r in state.modify_server({"name": "X", "url": "x"})]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_value_error(self) -> None:
        state = _StubMCPServerState()
        state.current_server = _server()
        [
            r
            async for r in state.modify_server(
                {"name": "X", "url": "x", "headers_json": "bad"}
            )
        ]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_exception(self) -> None:
        state = _StubMCPServerState()
        state.current_server = _server()
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            [r async for r in state.modify_server({"name": "X", "url": "x"})]
        assert state.loading is False


# ================================================================
# Delete server
# ================================================================


class TestDeleteServer:
    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _StubMCPServerState()
        s = _server()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=s)
            repo.delete_by_id = AsyncMock(return_value=True)
            repo.find_all_ordered_by_name = AsyncMock(return_value=[])
            [r async for r in state.delete_server(1)]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_not_found(self) -> None:
        state = _StubMCPServerState()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=None)
            [r async for r in state.delete_server(999)]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_delete_fails(self) -> None:
        state = _StubMCPServerState()
        s = _server()
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=s)
            repo.delete_by_id = AsyncMock(return_value=False)
            [r async for r in state.delete_server(1)]
        assert state.loading is False

    @pytest.mark.asyncio
    async def test_exception(self) -> None:
        state = _StubMCPServerState()
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            [r async for r in state.delete_server(1)]
        assert state.loading is False


# ================================================================
# Toggle active
# ================================================================


class TestToggleActive:
    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        s.name = "test"
        s.model_copy.return_value = MagicMock(id=1, name="test", active=False)
        state.servers = [s]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=s)
            repo.save = AsyncMock(return_value=s)
            [r async for r in state.toggle_server_active(1, False)]
        assert state.updating_active_server_id is None

    @pytest.mark.asyncio
    async def test_not_found(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        s.model_copy.return_value = MagicMock(id=1, active=False)
        state.servers = [s]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=None)
            [r async for r in state.toggle_server_active(1, False)]
        assert state.updating_active_server_id is None

    @pytest.mark.asyncio
    async def test_exception(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        s.model_copy.return_value = MagicMock(id=1, active=False)
        state.servers = [s]
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            [r async for r in state.toggle_server_active(1, False)]
        assert state.updating_active_server_id is None


# ================================================================
# Update role
# ================================================================


class TestUpdateRole:
    @pytest.mark.asyncio
    async def test_success(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        s.name = "test"
        s.required_role = None
        s.model_copy.return_value = MagicMock(id=1, required_role=None)
        state.servers = [s]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=s)
            repo.save = AsyncMock(return_value=s)
            [r async for r in state.update_server_role(1, "admin")]
        assert state.updating_role_server_id is None

    @pytest.mark.asyncio
    async def test_not_found(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        s.required_role = None
        s.model_copy.return_value = MagicMock(id=1, required_role=None)
        state.servers = [s]
        with (
            patch(
                f"{_PATCH}.get_asyncdb_session",
                return_value=_db_context(),
            ),
            patch(f"{_PATCH}.mcp_server_repo") as repo,
        ):
            repo.find_by_id = AsyncMock(return_value=None)
            [r async for r in state.update_server_role(1, "admin")]
        assert state.updating_role_server_id is None

    @pytest.mark.asyncio
    async def test_exception(self) -> None:
        state = _StubMCPServerState()
        s = MagicMock()
        s.id = 1
        s.required_role = None
        s.model_copy.return_value = MagicMock(id=1, required_role=None)
        state.servers = [s]
        with patch(
            f"{_PATCH}.get_asyncdb_session",
            side_effect=RuntimeError("db"),
        ):
            [r async for r in state.update_server_role(1, "admin")]
        assert state.updating_role_server_id is None
